"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginSettingsManager = void 0;
const vscode = require("vscode");
const defaultSettings_1 = require("./data/defaultSettings");
const fs = require("fs");
const os = require("os");
const path = require("path");
const DiagnsoticsManager_1 = require("./analyzer/DiagnsoticsManager");
const events_1 = require("events");
const AnalyzerUtils_1 = require("./analyzer/AnalyzerUtils");
const MessageSender_1 = require("./utilities/MessageSender");
class PluginSettingsManager {
    static get settings() {
        return JSON.parse(JSON.stringify(this._pluginSettings));
    }
    static saveSettings(newSettings, isNeedUpdating = false, savePath) {
        return __awaiter(this, void 0, void 0, function* () {
            if (isNeedUpdating) {
                this.settingsPreprocess(newSettings);
                yield this.updateSettings(newSettings);
            }
            this._pluginSettings = newSettings;
            yield this.writeSettings(this.settings, savePath);
            this._eventEmitter.emit("change", this.settings);
        });
    }
    static loadOrCreateSettings() {
        return __awaiter(this, void 0, void 0, function* () {
            const settings = yield this.loadSettings();
            this.saveSettings(settings, true);
        });
    }
    static addChangeEventListener(callback) {
        this._eventEmitter.on("change", callback);
    }
    static removeChangeEventListener(callback) {
        this._eventEmitter.removeListener("change", callback);
    }
    static findEnableDiagnostics(diagnostics) {
        const disabledDiagnostics = PluginSettingsManager._pluginSettings.disabledDiagnostics;
        return diagnostics.filter(d1 => !disabledDiagnostics.some(d2 => d1.code === d2.code));
    }
    static updateSettings(settings) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.updateSettingsRecursive(settings, settings, (0, defaultSettings_1.getDefaultSettings)());
        });
    }
    /**
     * Рекурсивное обновление структуры settings у пользователей, при ее изменении в default settings.
     * Предназначен для более удобной поддержки обратной совместимости
     * По умолчанию работает так:
     *
     * если в settings есть поле, которого нет в default settings - удаляет это поле;
     *
     * если в settings нет поля, которое есть в default settings - добавляет его со значением по-умолчанию;
     *
     * если и в settings и в default settings есть поле с одним названием, но с разным типом значения - присваивает ему значение из default settings;
     *
     * В методы changePropertyIfNecessary и deletePropertyIfNecessary можно вносить исключения в этот процесс, например если перед удалением поля,
     * на основе его значения, нужно создать новое поле (к примеру преобразование списка visibleDiagnostics в unvisibleRules)
     * @param settings настройки плагина
     * @param settingsItem урвоень вложенности в настройках плагина, который обрабатывается на текущей реализации
     * @param defaultSettingsItem соответсвующий settingsItem уровень вложенности из default settings
     * @param path путь к текущему уровню вложенности
     */
    static updateSettingsRecursive(settings, settingsItem, defaultSettingsItem, path = "") {
        return __awaiter(this, void 0, void 0, function* () {
            if (path === "") {
                path = "settings";
            }
            // Временный фикс для устранения появившихся дубликатов
            if (settings) {
                settings.disabledDiagnostics = Array.from(new Set(settings.disabledDiagnostics));
            }
            //
            for (var defaultPropertyName of Object.keys(defaultSettingsItem)) {
                let itemProperty = settingsItem[defaultPropertyName];
                const defaultItemProperty = defaultSettingsItem[defaultPropertyName];
                if (itemProperty === undefined) {
                    settingsItem[defaultPropertyName] = defaultItemProperty;
                }
                else if (typeof (itemProperty) === "object" && !Array.isArray(itemProperty)
                    && typeof (defaultItemProperty) === "object" && !Array.isArray(defaultItemProperty)) {
                    yield this.updateSettingsRecursive(settings, settingsItem[defaultPropertyName], defaultSettingsItem[defaultPropertyName], path + `.${defaultPropertyName}`);
                }
                else {
                    const newProperty = this.changePropertyIfNecessary(itemProperty, defaultItemProperty, path + `.${defaultPropertyName}`);
                    if (newProperty) {
                        settingsItem[defaultPropertyName] = newProperty;
                    }
                }
            }
            for (var propertyName of Object.keys(settingsItem)) {
                if (defaultSettingsItem[propertyName] === undefined) {
                    if ((yield this.deletePropertyIfNecessary(settings, path + `.${propertyName}`))) {
                        delete settingsItem[propertyName];
                    }
                }
            }
        });
    }
    static changePropertyIfNecessary(property, defaultProperty, path) {
        // добавлено в версии 7.27
        if (path === "settings.excludedPaths") {
            if (property.some((p) => p.includes("**"))) {
                property = property.map((p) => p.replace(/\*\*/g, "*"));
                return property;
            }
        }
        // добавлено в версии 7.28
        else if (path === "settings.columns") {
            property = property.map((col) => {
                delete col.title;
                return col;
            });
        }
        // проверка по умолчанию
        else if (Array.isArray(property) !== Array.isArray(defaultProperty)
            || isNaN(Number(property === "" ? "NaN" : property)) !== isNaN(Number(defaultProperty === "" ? "NaN" : defaultProperty))
            || this.isBoolean(property) !== this.isBoolean(defaultProperty)) {
            return defaultProperty;
        }
    }
    static deletePropertyIfNecessary(settings, path) {
        return __awaiter(this, void 0, void 0, function* () {
            // добавлено в версии 7.28
            if (path === "settings.analysis.timeout.selectValue") {
                const value = settings.analysis.timeout.selectValue;
                settings.analysis.timeout = value;
            }
            if (path === "settings.visibleDiagnostics") {
                const diagnostics = yield DiagnsoticsManager_1.DiagnosticsManager.getDiagnostics();
                const visibleDiagnostics = settings.visibleDiagnostics;
                const disabledDiagnostics = [...diagnostics.filter(rule => !visibleDiagnostics.some((vr) => vr.code.toUpperCase() === rule.code.toUpperCase()))];
                settings.disabledDiagnostics = disabledDiagnostics;
            }
            return true;
        });
    }
    static isBoolean(value) {
        const valueAsStr = value.toString().toLowerCase();
        return valueAsStr === "true" || valueAsStr === "false";
    }
    static writeSettings(settings, writePath) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const writableSettings = this.prepareSettingsForWriting(settings);
                const settingsPath = writePath !== null && writePath !== void 0 ? writePath : yield this.getSettingsPath();
                if (!settingsPath) {
                    return;
                }
                const folder = path.dirname(settingsPath);
                if (!fs.existsSync(folder)) {
                    yield fs.promises.mkdir(folder, { recursive: true });
                }
                yield fs.promises.writeFile(settingsPath, JSON.stringify(writableSettings, undefined, 2));
            }
            catch (e) {
                const message = `Can't create or update pvs-studio plugin settings file.`;
                vscode.window.showErrorMessage(message);
            }
        });
    }
    static prepareSettingsForWriting(settings) {
        const writableSettings = settings;
        writableSettings.analysis.timeout = settings.analysis.timeout.id;
        writableSettings.disabledDiagnostics = settings.disabledDiagnostics.map(d => d.code);
        return writableSettings;
    }
    static settingsPreprocess(settings) {
        if (settings.analysis) {
            if (typeof settings.analysis.timeout === "string") {
                settings.analysis.timeout = AnalyzerUtils_1.AnalyzerUtils.analysisTimeoutValues.find(t => t.id === settings.analysis.timeout);
            }
        }
        if (settings.disabledDiagnostics instanceof Array) {
            const disabledDiagnostics = [];
            const filters = { codes: settings.disabledDiagnostics.filter((item) => typeof item === "string") };
            disabledDiagnostics.push(...DiagnsoticsManager_1.DiagnosticsManager.getDiagnostics(filters));
            settings.disabledDiagnostics = disabledDiagnostics;
        }
    }
    static getSettingsPath() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._settingsPath === undefined) {
                const settingsFileName = "vscode_settings.json";
                const userHomeDir = os.homedir();
                const platform = os.platform();
                if (platform === "linux" || platform === "darwin") {
                    this._settingsPath = path.join(userHomeDir, ".config", "PVS-Studio", settingsFileName);
                }
                else if (platform === "win32") {
                    this._settingsPath = path.join(userHomeDir, "AppData", "Roaming", "PVS-Studio", settingsFileName);
                    yield this.crutchForChangingConfigDirectory(userHomeDir, this._settingsPath);
                }
                if (this._settingsPath === undefined) {
                    const message = "An unsupported operating system was detected. PVS-Studio analyzer works only on Linux, Windows or MacOS.";
                    vscode.window.showErrorMessage(message);
                    throw Error(message);
                }
            }
            return this._settingsPath;
        });
    }
    /**
     * FIXME кастыль, необходимый из-за того, что файл с конфигурацией плагина сохраняется не в той директории. Переносит файл в нужную директорию
     */
    static crutchForChangingConfigDirectory(userHomeDir, rightSettingsPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const wrongDirectory = path.join(userHomeDir, "AppData", "PVS-Studio");
            if (fs.existsSync(wrongDirectory)) {
                const settingsFileName = path.basename(rightSettingsPath);
                const wrongSettingsPath = path.join(wrongDirectory, settingsFileName);
                if (fs.existsSync(wrongSettingsPath)) {
                    const settingsFile = yield fs.promises.readFile(wrongSettingsPath);
                    const rightDirectory = path.dirname(rightSettingsPath);
                    if (!fs.existsSync(rightDirectory)) {
                        yield fs.promises.mkdir(rightDirectory, { recursive: true });
                    }
                    yield fs.promises.writeFile(rightSettingsPath, settingsFile.toString());
                }
                yield fs.promises.rmdir(wrongDirectory, { recursive: true });
            }
        });
    }
    static loadSettings() {
        return __awaiter(this, void 0, void 0, function* () {
            let settings;
            const settingsPath = yield this.getSettingsPath();
            const settingsFileIsExists = fs.existsSync(settingsPath);
            const pluginConfig = vscode.workspace.getConfiguration("pvsStudio");
            try {
                if (settingsFileIsExists) {
                    const settingsJson = (yield fs.promises.readFile(settingsPath)).toString();
                    settings = JSON.parse(settingsJson);
                }
                else {
                    if (pluginConfig.settings) {
                        settings = JSON.parse(JSON.stringify(pluginConfig.settings));
                        pluginConfig.update("settings", undefined, true);
                    }
                    else {
                        settings = (0, defaultSettings_1.getDefaultSettings)();
                    }
                }
            }
            catch (_a) {
                const message = `Can't read pvs-studio plugin settings file. The default settings will be used.`;
                MessageSender_1.MessageSender.send(message, "warning");
                settings = (0, defaultSettings_1.getDefaultSettings)();
            }
            if (pluginConfig.settings) {
                // удаляем копию настроек из конфига рабочего пространства, так как теперь настройки хранятся только в vscode_settings.json,
                // а работа с ними осуществляется через текущий класс.
                pluginConfig.update("settings", undefined, true);
            }
            return settings;
        });
    }
}
exports.PluginSettingsManager = PluginSettingsManager;
PluginSettingsManager._eventEmitter = new events_1.EventEmitter();
PluginSettingsManager.extensionId = 'EvgeniyRyzhkov.pvs-studio-vscode';
//# sourceMappingURL=PluginSettingsManager.js.map