"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AnalyzerWithLicenseActions = exports.Analyzer = void 0;
const os = require("os");
const vscode = require("vscode");
const child_process_1 = require("child_process");
const fs = require("fs");
const path = require("path");
const saveFile_1 = require("../utilities/saveFile");
const AnalyzerUtils_1 = require("./AnalyzerUtils");
const PluginSettingsManager_1 = require("../PluginSettingsManager");
const openLinkInBrowser_1 = require("../utilities/openLinkInBrowser");
const MessageSender_1 = require("../utilities/MessageSender");
class Analyzer {
    constructor() {
        this._analyzerInstallLink = "https://pvs-studio.com/en/pvs-studio/download/";
        this._analyzerIsNoIsntalledMessage = `The core of the PVS-Studio analyzer is not installed or not found. In the second case, please specify the path to the core in the extension settings.`;
        this._notAllSelectedFilesWillBeAnalyzedMessage = `Among the selected files, only those with one of the following extensions will be analyzed:`;
        this.timerValueToSend = 20 * 1000;
        this.bufferLengthToOutput = 25000;
        this._analysisEndedSwitch = false;
        this.platform = os.platform();
    }
    get isRunAnalysis() {
        var _a;
        if (((_a = this._analysis) === null || _a === void 0 ? void 0 : _a.exitCode) === null) {
            return true;
        }
        else {
            return false;
        }
    }
    onRunningExtension() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    runAnalysis(reportPanel, selectedFiles) {
        return __awaiter(this, void 0, void 0, function* () {
            const filesToAnalysis = selectedFiles.filter(p => this.description.extensionsOfAnalyzedFiles.includes(path.extname(p)));
            const hasFilesToAnalysis = filesToAnalysis.length > 0;
            if (hasFilesToAnalysis && filesToAnalysis.length < selectedFiles.length) {
                const filesExtensions = [...new Set(filesToAnalysis.map(f => path.extname(f)))];
                let message = `${this._notAllSelectedFilesWillBeAnalyzedMessage} ${filesExtensions[0]}`;
                for (var i = 1; i < filesExtensions.length; i++) {
                    message += `, ${filesExtensions[i]}`;
                }
                message += ".";
                MessageSender_1.MessageSender.send(message, "warning");
            }
            yield vscode.commands.executeCommand("PVSPanel.focus");
            const process = yield this.initializeAnalysisProcess(reportPanel, () => this.onAnalysisStarted(reportPanel), () => this.isAnalysisEnded(reportPanel.view), hasFilesToAnalysis ? filesToAnalysis : undefined);
            if (!process) {
                reportPanel.view.webview.postMessage({
                    type: "analysisFailed",
                });
            }
            return process;
        });
    }
    endAnalysis(reportPanel) {
        var _a, _b;
        (_a = this._analysis) === null || _a === void 0 ? void 0 : _a.stdin.write(this.endAnalysisKey);
        (_b = this._analysis) === null || _b === void 0 ? void 0 : _b.stdin.end();
        this._analysisEndedSwitch = true;
    }
    suppressWarnings(warnings, reportPanel) {
        return __awaiter(this, void 0, void 0, function* () {
            if (warnings.find((w) => w.falseAlarm)) {
                MessageSender_1.MessageSender.send(`Some of the selected messages are marked as False Alarms in the source code. Remove the False Alarm mark to suppress them.`, "warning");
            }
            warnings = warnings.filter(w => !w.falseAlarm);
            if (warnings.length > 0) {
                yield vscode.commands.executeCommand("PVSPanel.focus");
                return yield this.initializeSuppressionProcess(warnings, reportPanel);
            }
        });
    }
    isAnalyzerInstalled() {
        return __awaiter(this, void 0, void 0, function* () {
            const analyzerPath = yield this.getAnalyzerPath();
            if (!analyzerPath.isSupported) {
                return false;
            }
            const isInstall = fs.existsSync(analyzerPath.path);
            if (!isInstall) {
                new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                    const response = yield MessageSender_1.MessageSender.send(this._analyzerIsNoIsntalledMessage, "error", false, "Install", "Edit setting");
                    if (response === "Install") {
                        (0, openLinkInBrowser_1.default)(this._analyzerInstallLink);
                    }
                    else if (response === "Edit setting") {
                        vscode.commands.executeCommand("workbench.action.openSettings", "pvsStudio");
                    }
                    resolve();
                }));
            }
            return isInstall;
        });
    }
    processSuppression(command, args, view, warnings, targetSuppressionFilePath, errorLogFilePath, processOptions = undefined) {
        return __awaiter(this, void 0, void 0, function* () {
            const suppressProcess = (0, child_process_1.spawn)(command, args, processOptions !== null && processOptions !== void 0 ? processOptions : {
                cwd: yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath()
            });
            suppressProcess.stdout.setEncoding("utf8");
            let outputLog = "";
            suppressProcess.stdout.on("data", (data) => {
                outputLog += data.toString();
            });
            let errorLog = "";
            suppressProcess.stderr.on("data", (data) => {
                errorLog += data.toString();
            });
            suppressProcess.on("close", (code) => __awaiter(this, void 0, void 0, function* () {
                yield AnalyzerUtils_1.AnalyzerUtils.removeTempFolder();
                if (code === 1 || code === 512) {
                    (0, saveFile_1.saveFile)(errorLog, errorLogFilePath);
                    MessageSender_1.MessageSender.send(`Unable to suppress analyzer messages. Detailed log file was saved to ${errorLogFilePath}`, "error");
                }
                else if (code === 0) {
                    view.webview.postMessage({
                        type: "clearSuppressedWarnings",
                        warnings: warnings,
                    });
                    MessageSender_1.MessageSender.send(`Suppressing analyzer messages is finished. Suppress file was saved to ${targetSuppressionFilePath}.`, "info");
                }
            }));
            return suppressProcess;
        });
    }
    processAnalysis(command, args, output, view, processOptions = undefined, analysisProgressHandler = undefined) {
        return __awaiter(this, void 0, void 0, function* () {
            this._analysis = (0, child_process_1.spawn)(command, args, processOptions !== null && processOptions !== void 0 ? processOptions : {
                cwd: yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath()
            });
            const sourceTreeRoot = PluginSettingsManager_1.PluginSettingsManager.settings.sourceTreeRoot;
            this._analysis.stdout.setEncoding("utf8");
            this.startTimer(output, view, sourceTreeRoot);
            let allOutput = "";
            let allErrors = "";
            let buffer = "";
            let isAnalysisStarted = false;
            this._analysis.stdout.on("data", (chunk) => {
                buffer += chunk.toString();
                if (buffer.length >= this.bufferLengthToOutput || !isAnalysisStarted) {
                    output.setAnalysisOutput(buffer, (newProgress) => {
                        if (analysisProgressHandler) {
                            newProgress = analysisProgressHandler(newProgress);
                        }
                        if (newProgress > 0) {
                            isAnalysisStarted = true;
                        }
                        view.webview.postMessage({
                            type: "analyzerProgressMessage",
                            message: "",
                            percentages: newProgress,
                        });
                    });
                    allOutput += buffer;
                    buffer = "";
                }
            });
            this._analysis.on("close", (code) => __awaiter(this, void 0, void 0, function* () {
                allOutput += buffer;
                output.setAnalysisOutput(buffer);
                this.sendWarningsToView(output, view, sourceTreeRoot, false);
                this._analysisEndedSwitch = false;
                if (PluginSettingsManager_1.PluginSettingsManager.settings.analysis.removeIntermediateFiles) {
                    yield AnalyzerUtils_1.AnalyzerUtils.removeTempFolder();
                }
                view.webview.postMessage({
                    type: "analysisEnded",
                });
                let isNeedCodeOutput = true;
                if (this.platform === "win32") {
                    isNeedCodeOutput = ![0, 256, 1024].includes(code);
                }
                else {
                    isNeedCodeOutput = ![0, 4, 8].includes(code);
                }
                if (isNeedCodeOutput) {
                    MessageSender_1.MessageSender.send(`Analysis process exited with code ${code}.`, "error");
                }
                this._analysis = undefined;
            }));
            if (this.description.analyzerName !== "Java projects analyzer") {
                this._analysis.stderr.on("data", (data) => {
                    const chunks = data.toString().split(/\r\n?|\n\r?/gm);
                    const noJsonChunks = [];
                    for (var i = 0; i < chunks.length; i++) {
                        var chunk = chunks[i].trim();
                        if (chunk.length === 0) {
                            continue;
                        }
                        if (chunk.startsWith('{') && chunk.endsWith('}')) {
                            output.setAnalysisOutput(chunk);
                        }
                        else {
                            noJsonChunks.push(chunk);
                        }
                    }
                    if (noJsonChunks.length > 0) {
                        const gridError = AnalyzerUtils_1.AnalyzerUtils.toGridMessage(noJsonChunks.join('\r\n'));
                        output.setAnalysisOutput(JSON.stringify(gridError));
                    }
                });
            }
            return this._analysis;
        });
    }
    isAnalysisEnded(view) {
        if (this._analysisEndedSwitch) {
            view.webview.postMessage({
                type: "analysisEnded",
            });
            this._analysisEndedSwitch = false;
            return true;
        }
        return false;
    }
    onAnalysisStarted(reportPanel) {
        reportPanel.view.webview.postMessage({
            type: "analyzerProgressStart",
        });
        reportPanel.view.webview.postMessage({
            type: "analysisStarted",
        });
    }
    startTimer(analyzerOutput, view, sourceTreeRoot) {
        this._analysisTimer = setTimeout(() => {
            this.sendWarningsToView(analyzerOutput, view, sourceTreeRoot);
        }, this.timerValueToSend);
    }
    stopTimer() {
        clearTimeout(this._analysisTimer);
    }
    sendWarningsToView(analyzerOutput, view, sourceTreeRoot, needRestartTimer = true) {
        this.stopTimer();
        if (needRestartTimer) {
            this.startTimer(analyzerOutput, view, sourceTreeRoot);
        }
        view.webview.postMessage({
            type: "analyzerProgressWarnings",
            warnings: AnalyzerUtils_1.AnalyzerUtils.prepareWarnings(analyzerOutput.warningsBuffer)
        });
        analyzerOutput.clearWarningsBuffer();
    }
    checkAnalyzerVersion(version) {
        var _a;
        const analyzerVersionStr = version.slice(0, 2).join("");
        const pluginVersion = (_a = vscode.extensions.getExtension(PluginSettingsManager_1.PluginSettingsManager.extensionId)) === null || _a === void 0 ? void 0 : _a.packageJSON.version.split('.').slice(0, 2).join('');
        const isDebug = analyzerVersionStr === "00" || pluginVersion === "00";
        return { isSupported: isDebug || analyzerVersionStr === pluginVersion, isDebug: isDebug };
    }
    ;
    static execShellCommand(cmd) {
        const exec = require('child_process').exec;
        return new Promise((resolve, reject) => {
            exec(cmd, (error, stdout, stderr) => {
                if (error) {
                    console.warn(error);
                }
                resolve(stdout ? stdout : stderr);
            });
        });
    }
    checkAnalyzerVersionWithExecution(analyzerCallable, isWindows = false, isCmake = false) {
        return __awaiter(this, void 0, void 0, function* () {
            let analyzerVersion = [];
            let command = '';
            if (isWindows) {
                command = `"${analyzerCallable}" --version`;
            }
            else {
                command = `${analyzerCallable} --version`;
            }
            let stdout = yield Analyzer.execShellCommand(command);
            if (isCmake) {
                analyzerVersion = stdout.toString().slice(11).split('.').map((x) => Number(x)).slice(0, 2);
            }
            else {
                analyzerVersion = stdout.toString().split('.').map((x) => Number(x)).slice(0, 2);
            }
            return this.checkAnalyzerVersion(analyzerVersion).isSupported;
        });
    }
}
exports.Analyzer = Analyzer;
class AnalyzerWithLicenseActions extends Analyzer {
}
exports.AnalyzerWithLicenseActions = AnalyzerWithLicenseActions;
//# sourceMappingURL=Analyzer.js.map