"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AnalyzerManager = void 0;
const Analyzer_1 = require("./Analyzer");
const CMakeAnalyzer_1 = require("./CMakeAnalyzer/CMakeAnalyzer");
const MSBuildAnalyzer_1 = require("./MSBuildAnalyzer/MSBuildAnalyzer");
const vscode = require("vscode");
const fs = require("fs");
const path = require("path");
const xml2js = require("xml2js");
const JavaAnalyzer_1 = require("./JavaAnalyzer/JavaAnalyzer");
const MessageSender_1 = require("../utilities/MessageSender");
var SuppressionMode;
(function (SuppressionMode) {
    SuppressionMode[SuppressionMode["suppressAll"] = 0] = "suppressAll";
    SuppressionMode[SuppressionMode["suppressFiltered"] = 1] = "suppressFiltered";
})(SuppressionMode || (SuppressionMode = {}));
;
class AnalyzerManager {
    static warningsSuppressionDisable() {
        this._isWarningsSuppressionAllowed = false;
    }
    static onRunningExtension() {
        return __awaiter(this, void 0, void 0, function* () {
            const specificExtArray = this.getExtensionsOfAnalyzedFiles();
            if (specificExtArray.length > 0) {
                yield vscode.commands.executeCommand("setContext", "pvs-studio.extensionsOfAnalyzedFiles", specificExtArray);
            }
            for (var analyzer of this.allAnalyzers) {
                const isAnalysisPossible = yield analyzer.isAnalysisPossible();
                if (isAnalysisPossible) {
                    yield analyzer.onRunningExtension();
                }
            }
        });
    }
    static runAnalysis(reportPanel, isintermodular = undefined, files = []) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (reportPanel.isBlocked) {
                    MessageSender_1.MessageSender.send("Another process is already running.", "warning");
                    return;
                }
                reportPanel.blockingPanel(true);
                const existFiles = files.filter(f => fs.existsSync(f));
                const selectedFiles = yield this.getFilesFromDirectoriesRecursive(existFiles);
                this._selectedAnalyzer = yield this.selectAnalyzer(reportPanel.view, selectedFiles);
                if (this._selectedAnalyzer) {
                    if ("intermodularAnalysisIsEnabled" in this._selectedAnalyzer) {
                        this._selectedAnalyzer.intermodularAnalysisIsEnabled = isintermodular;
                    }
                    const process = yield this._selectedAnalyzer.runAnalysis(reportPanel, selectedFiles);
                    if (process === undefined) {
                        reportPanel.blockingPanel(false);
                    }
                    else {
                        process.on("close", _ => {
                            reportPanel.blockingPanel(false);
                        });
                    }
                    if (this._selectedAnalyzer.isRunAnalysis) {
                        this._isWarningsSuppressionAllowed = true;
                    }
                }
                else {
                    reportPanel.blockingPanel(false);
                }
            }
            catch (_a) {
                MessageSender_1.MessageSender.send("Unable to start the analysis.", "error");
                reportPanel.blockingPanel(false);
            }
        });
    }
    static isRunAnalysis() {
        return this._selectedAnalyzer && this._selectedAnalyzer.isRunAnalysis;
    }
    static endAnalysis(reportPanel) {
        var _a;
        (_a = this._selectedAnalyzer) === null || _a === void 0 ? void 0 : _a.endAnalysis(reportPanel);
    }
    static suppressWarnings(warnings, reportPanel) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (reportPanel.isBlocked) {
                    MessageSender_1.MessageSender.send("Another process is already running.", "warning");
                    return;
                }
                if (this._selectedAnalyzer === undefined || !this._isWarningsSuppressionAllowed) {
                    MessageSender_1.MessageSender.send("Suppression of warnings is available only after analyzing.", "error");
                    return;
                }
                const analyzer = this._selectedAnalyzer;
                if (!analyzer) {
                    return;
                }
                reportPanel.blockingPanel(true);
                const process = yield analyzer.suppressWarnings(warnings, reportPanel);
                if (process) {
                    process.on("close", _ => {
                        reportPanel.blockingPanel(false);
                    });
                }
                else {
                    reportPanel.blockingPanel(false);
                }
            }
            catch (e) {
                MessageSender_1.MessageSender.send(e.message, "error");
                reportPanel.blockingPanel(false);
            }
        });
    }
    static requestSuppressionWarrnings(reportPanel) {
        return __awaiter(this, void 0, void 0, function* () {
            if (reportPanel.isBlocked) {
                MessageSender_1.MessageSender.send("Another process is already running.", "warning");
                return;
            }
            if (this._selectedAnalyzer === undefined || !this._isWarningsSuppressionAllowed) {
                MessageSender_1.MessageSender.send("Suppression of warnings is available only after analyzing.", "error");
                return;
            }
            const suppressionMode = yield this.selectSuppressionMode();
            if (suppressionMode === undefined) {
                return;
            }
            if (suppressionMode === SuppressionMode.suppressAll) {
                reportPanel.view.webview.postMessage({ type: "getAllWarningsForSuppress" });
            }
            else if (suppressionMode === SuppressionMode.suppressFiltered) {
                reportPanel.view.webview.postMessage({ type: "getFilteredWarningsForSuppress" });
            }
        });
    }
    static getExtensionsOfAnalyzedFiles() {
        const extensions = [];
        for (var a of this.allAnalyzers) {
            if (a.description.extensionsOfAnalyzedFiles) {
                extensions.push(...a.description.extensionsOfAnalyzedFiles);
            }
        }
        return extensions;
    }
    static getFilesFromDirectoriesRecursive(paths) {
        return __awaiter(this, void 0, void 0, function* () {
            const files = [];
            for (var p of paths) {
                if ((yield fs.promises.lstat(p)).isDirectory()) {
                    const directoryItemPaths = (yield fs.promises.readdir(p, { withFileTypes: true })).map(item => path.join(p, item.name));
                    files.push(...(yield this.getFilesFromDirectoriesRecursive(directoryItemPaths)));
                }
                else {
                    files.push(p);
                }
            }
            return files;
        });
    }
    static selectSuppressionMode() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise(resolve => {
                const quickPick = vscode.window.createQuickPick();
                const suppressAllItem = { label: "Suppress all", detail: "Suppress all messages, that also includes all the messages that are currently invisible because of the active filters." };
                const suppressFilteredItem = { label: "Suppress filtered", detail: "Suppress only messages visible in the grid." };
                quickPick.items = [suppressAllItem, suppressFilteredItem];
                quickPick.title = "Select the suppression mode";
                quickPick.onDidChangeSelection(e => {
                    if (e[0].label === suppressAllItem.label) {
                        resolve(SuppressionMode.suppressAll);
                    }
                    else {
                        resolve(SuppressionMode.suppressFiltered);
                    }
                    quickPick.dispose();
                });
                quickPick.onDidHide(_ => {
                    quickPick.dispose();
                    resolve(undefined);
                });
                quickPick.show();
            });
        });
    }
    static selectAnalyzer(view, selectedFiles = []) {
        return __awaiter(this, void 0, void 0, function* () {
            let analyzersUsability = new Map();
            for (var analyzer of this.allAnalyzers) {
                const isUsableAnalyzer = yield analyzer.isAnalysisPossible();
                analyzersUsability.set(analyzer, isUsableAnalyzer);
            }
            let usableAnalyzers = Array.from(analyzersUsability.keys()).filter(analyzer => analyzersUsability.get(analyzer) === true);
            if (usableAnalyzers.length > 0 && selectedFiles.length > 0) {
                yield new Promise(resolve => {
                    const filesExtensions = [...new Set(selectedFiles.map(f => path.extname(f)))];
                    const analyzerAndUsabilityPairs = usableAnalyzers.map((a) => ({ analyzer: a, usability: a.description.extensionsOfAnalyzedFiles.some((ext) => filesExtensions.includes(ext)) }));
                    usableAnalyzers = analyzerAndUsabilityPairs.filter(pair => pair.usability).map(pair => pair.analyzer);
                    resolve();
                });
                if (usableAnalyzers.length === 0) {
                    MessageSender_1.MessageSender.send(this._notSupportedSelectedFilesMessage, "error");
                    return;
                }
            }
            let selectedAnalyzer = undefined;
            if (usableAnalyzers.length === 1) {
                selectedAnalyzer = usableAnalyzers[0];
            }
            else if (usableAnalyzers.length > 1) {
                if (selectedFiles.length > 0) {
                    const filesExtensions = [...new Set(selectedFiles.map(f => path.extname(f)))];
                    const analyzersPriority = usableAnalyzers.map((a) => ({ analyzer: a, priority: a.description.extensionsOfAnalyzedFiles.filter((ext) => filesExtensions.includes(ext)).length }));
                    const maxPriority = Math.max(...analyzersPriority.map(pair => pair.priority));
                    selectedAnalyzer = analyzersPriority.find(a => a.priority === maxPriority).analyzer;
                }
                else {
                    const analyzerAndMessagePairs = usableAnalyzers.map(analyzer => ({
                        analyzer: analyzer,
                        message: this._analyzerTypeMessage.replace(`{analyzerType}`, analyzer.description.analyzedProjectsType)
                    }));
                    const selectedAnalyzerType = yield vscode.window.showQuickPick(analyzerAndMessagePairs.map(pair => pair.message), { placeHolder: this._analyzerSelectHint });
                    selectedAnalyzer = analyzerAndMessagePairs.find(pair => pair.message === selectedAnalyzerType).analyzer;
                }
            }
            else {
                let errorMessage = this._noProjectFoundForAnalysisMessage;
                for (var i = 0; i < this.allAnalyzers.length; i++) {
                    const analyzer = this.allAnalyzers[i];
                    errorMessage += `\n${i + 1}. ${analyzer.description.conditionForAnalysisPossibility}`;
                    errorMessage += i === this.allAnalyzers.length - 1 ? "." : ";";
                }
                MessageSender_1.MessageSender.send(errorMessage, "error", true);
            }
            const hasAnalyzer = selectedAnalyzer && (yield selectedAnalyzer.isAnalyzerInstalled());
            if (hasAnalyzer) {
                return selectedAnalyzer;
            }
        });
    }
    static updateLicense(userName, licenseKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const analyzer = yield this.getAnalyzerWithLicenseActions();
            if (/^-{1,2}\S+$/.test(userName) || /^-{1,2}\S+$/.test(licenseKey)) {
                return { code: 1, output: ["Incorrect registration info."] };
            }
            if (!analyzer) {
                return;
            }
            return yield analyzer.activateLicense(userName, licenseKey);
        });
    }
    static getLicenseInfo() {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        return __awaiter(this, void 0, void 0, function* () {
            MessageSender_1.MessageSender.disable();
            const analyzer = yield this.getAnalyzerWithLicenseActions();
            MessageSender_1.MessageSender.enable();
            if (!analyzer) {
                return;
            }
            const { licenseInfo, licenseFilePath } = yield analyzer.getLicenseInfo();
            if (licenseFilePath) {
                const licenseFileText = (yield fs.promises.readFile(licenseFilePath)).toString();
                const ext = path.extname(licenseFilePath);
                let userName = "";
                let licenseKey = "";
                if (ext === ".lic") {
                    const licenseFileLines = licenseFileText.split(/\r\n?|\n\r?/gm);
                    userName = (_b = (_a = licenseFileLines[0]) === null || _a === void 0 ? void 0 : _a.trim()) !== null && _b !== void 0 ? _b : "";
                    licenseKey = (_d = (_c = licenseFileLines[1]) === null || _c === void 0 ? void 0 : _c.trim()) !== null && _d !== void 0 ? _d : "";
                }
                else if (ext === ".xml") {
                    const settingsXml = yield xml2js.parseStringPromise(licenseFileText).catch(err => {
                        const error = err;
                        if (error) {
                            MessageSender_1.MessageSender.send(error.message, "error");
                        }
                    });
                    userName = (_g = (_f = (_e = settingsXml["ApplicationSettings"]) === null || _e === void 0 ? void 0 : _e["UserName"]) === null || _f === void 0 ? void 0 : _f[0]) !== null && _g !== void 0 ? _g : "";
                    licenseKey = (_k = (_j = (_h = settingsXml["ApplicationSettings"]) === null || _h === void 0 ? void 0 : _h["SerialNumber"]) === null || _j === void 0 ? void 0 : _j[0]) !== null && _k !== void 0 ? _k : "";
                }
                return { data: licenseInfo, userName: userName.trim(), licenseKey: licenseKey.trim() };
            }
        });
    }
    static getAnalyzerWithLicenseActions() {
        return __awaiter(this, void 0, void 0, function* () {
            const targetAnalyzers = this.allAnalyzers.filter(a => a instanceof Analyzer_1.AnalyzerWithLicenseActions);
            let selectedAnalyzer = undefined;
            MessageSender_1.MessageSender.disable();
            for (var i = 0; i < targetAnalyzers.length; i++) {
                if (yield targetAnalyzers[i].isAnalyzerInstalled()) {
                    selectedAnalyzer = targetAnalyzers[i];
                    break;
                }
            }
            MessageSender_1.MessageSender.enable();
            /**
             * Повторно выполняем функцию проверки на инсталировнность при включенном MessageSender, чтобы вывести все сообщения об ошибках только в том случае,
             * если найти подходящее установленное ядро не удалось.
             */
            if (!selectedAnalyzer) {
                for (var analyzer of targetAnalyzers) {
                    if (!(yield analyzer.isAnalyzerInstalled())) {
                        break;
                    }
                }
            }
            return selectedAnalyzer;
        });
    }
}
exports.AnalyzerManager = AnalyzerManager;
AnalyzerManager.allAnalyzers = [new CMakeAnalyzer_1.CMakeAnalyzer(), new MSBuildAnalyzer_1.MSBuildAnalyzer(), new JavaAnalyzer_1.JavaAnalyzer()];
AnalyzerManager._analyzerTypeMessage = `Analyzer for {analyzerType} projects`;
AnalyzerManager._analyzerSelectHint = "Several supported project types were found in the active directory. Select which analyzer you want to use.";
AnalyzerManager._analyzerSelectMessage = "Please select the project type to analyze in the command palette.";
AnalyzerManager._noProjectFoundForAnalysisMessage = `No suitable targets were found for analysis. Please open the folder corresponding to one of the following conditions:`;
AnalyzerManager._notSupportedSelectedFilesMessage = `No source or project files were found for analysis.`;
AnalyzerManager._isWarningsSuppressionAllowed = false;
//# sourceMappingURL=AnalyzerManager.js.map