"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AnalyzerUtils = void 0;
const vscode = require("vscode");
const path = require("path");
const fs = require("fs");
const hash_1 = require("../utilities/hash");
const os = require("os");
const MessageSender_1 = require("../utilities/MessageSender");
const PluginSettingsManager_1 = require("../PluginSettingsManager");
class AnalyzerUtils {
    static prepareWarnings(warnings) {
        var _a, _b;
        const processedWarnings = [];
        const sriMappingUri = vscode.Uri.joinPath(vscode.extensions.getExtension("EvgeniyRyzhkov.pvs-studio-vscode").extensionUri, "webview-settings", "build", "data", "srimap.json");
        const sriMapping = PluginSettingsManager_1.PluginSettingsManager.settings.analysis.securityRelatedIssues === true ? require(sriMappingUri.fsPath) : {};
        for (var warning of warnings) {
            const newPositions = [];
            for (var position of warning.positions) {
                if (position.hasOwnProperty("lines")) {
                    for (var line of position.lines) {
                        const newPosition = {};
                        newPosition.column = position.column ? position.column : 1;
                        newPosition.endColumn = position.endColumn ? position.endColumn : 1;
                        newPosition.line = line;
                        newPosition.endLine = line;
                        newPosition.file = position.file;
                        newPositions.push(newPosition);
                    }
                }
                else {
                    newPositions.push(position);
                }
            }
            if (newPositions.length > 0) {
                newPositions[0].navigation = (_a = newPositions[0].navigation) !== null && _a !== void 0 ? _a : {};
                const toInt32 = (value) => {
                    return Number(BigInt.asIntN(32, BigInt(value)));
                };
                newPositions[0].navigation.currentLine = newPositions[0].navigation.currentLine ? toInt32(newPositions[0].navigation.currentLine)
                    : warning.currLine ? toInt32((0, hash_1.getHashSumFromString)(warning.currLine))
                        : warning.currLineHash ? toInt32(warning.currLineHash)
                            : 0;
                newPositions[0].navigation.previousLine = newPositions[0].navigation.previousLine ? toInt32(newPositions[0].navigation.previousLine)
                    : warning.prevLine ? toInt32((0, hash_1.getHashSumFromString)(warning.prevLine))
                        : warning.prevLineHash ? toInt32(warning.prevLineHash)
                            : 0;
                newPositions[0].navigation.nextLine = newPositions[0].navigation.nextLine ? toInt32(newPositions[0].navigation.nextLine)
                    : warning.nextLine ? toInt32((0, hash_1.getHashSumFromString)(warning.nextLine))
                        : warning.nextLineHash ? toInt32(warning.nextLineHash)
                            : 0;
                newPositions[0].navigation.columns = 0;
            }
            let processedWarning = Object.assign(Object.assign({}, warning), { favorite: (_b = warning.favorite) !== null && _b !== void 0 ? _b : false, positions: newPositions });
            if (sriMapping[processedWarning.code] !== undefined) {
                sriMapping[processedWarning.code].forEach((element) => {
                    const sriLabel = "SEC-" + element;
                    if (!processedWarning.sastId.includes(sriLabel)) {
                        if (processedWarning.sastId.length > 0) {
                            processedWarning.sastId += ", ";
                        }
                        processedWarning.sastId += sriLabel;
                    }
                });
            }
            processedWarning.projects = !processedWarning.projects ? []
                : !Array.isArray(processedWarning.projects) ? [processedWarning.projects]
                    : processedWarning.projects;
            delete processedWarning.nextLine;
            delete processedWarning.currLine;
            delete processedWarning.prevLine;
            delete processedWarning.currLineHash;
            delete processedWarning.prevLineHash;
            delete processedWarning.nextLineHash;
            processedWarnings.push(processedWarning);
        }
        return processedWarnings;
    }
    static toGridMessage(message) {
        return {
            code: "",
            currLine: "",
            falseAlarm: false,
            level: 0,
            message: message,
            nextLine: "",
            positions: [],
        };
    }
    static getPVSConfigFilePaths(withGlobal = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const pvsFolderPath = (yield AnalyzerUtils.getPvsFolderPath());
            const files = yield fs.promises.readdir(pvsFolderPath);
            const pvsconfigPaths = files.filter(p => path.extname(p) === ".pvsconfig");
            if (withGlobal) {
                let pvsFolderPath;
                if (os.platform() === "win32") {
                    const appDataDir = process.env.APPDATA;
                    pvsFolderPath = path.join(appDataDir, "PVS-Studio");
                }
                else {
                    const homeDir = os.homedir();
                    pvsFolderPath = path.join(homeDir, ".config", "PVS-Studio");
                }
                let isFolderExists;
                try {
                    isFolderExists = yield fs.promises.access(pvsFolderPath, fs.constants.F_OK);
                }
                catch (error) {
                    isFolderExists = false;
                }
                const files = yield fs.promises.readdir(pvsFolderPath);
                pvsconfigPaths.push(...files.filter(p => path.extname(p) === ".pvsconfig"));
            }
            return pvsconfigPaths;
        });
    }
    static removeTempFolder() {
        return __awaiter(this, void 0, void 0, function* () {
            const tempFolderPath = yield this.getTempFolderPath();
            if (!fs.existsSync(tempFolderPath)) {
                return;
            }
            try {
                yield fs.promises.rmdir(tempFolderPath, {
                    "recursive": true
                });
            }
            catch (_a) {
                MessageSender_1.MessageSender.send("Failed to delete directory with temporary files: " + tempFolderPath + ".", "warning");
            }
        });
    }
    static getRootFolder() {
        var _a;
        let rootFolder = (_a = vscode.workspace.workspaceFolders) === null || _a === void 0 ? void 0 : _a[0];
        if (rootFolder) {
            return rootFolder;
        }
        const rootFileUri = vscode.workspace.workspaceFile;
        if (rootFileUri) {
            rootFolder = vscode.workspace.getWorkspaceFolder(vscode.Uri.file(path.dirname(rootFileUri.fsPath)));
            if (rootFolder) {
                return rootFolder;
            }
        }
    }
    static getTempFolderPath(createIfNoExists = true) {
        return __awaiter(this, void 0, void 0, function* () {
            const pvsFolderPath = yield this.getPvsFolderPath(createIfNoExists);
            if (!pvsFolderPath) {
                return "";
            }
            const tempFolderPath = path.join(pvsFolderPath, "Temp");
            if (createIfNoExists && !fs.existsSync(tempFolderPath)) {
                yield fs.promises.mkdir(tempFolderPath, { recursive: true });
            }
            return tempFolderPath;
        });
    }
    static getWarningsToSuppressionFilePath() {
        return __awaiter(this, void 0, void 0, function* () {
            const tempFolderPath = yield this.getTempFolderPath();
            return path.join(tempFolderPath, this.warningsToSuppressionFileName);
        });
    }
    static getPvsFolderPath(createIfNoExists = true) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const rootFolderPath = (_a = this.getRootFolder()) === null || _a === void 0 ? void 0 : _a.uri.fsPath;
            if (!rootFolderPath) {
                return "";
            }
            const pvsFolderPath = path.join(rootFolderPath, ".PVS-Studio");
            let exists = fs.existsSync(pvsFolderPath);
            if (createIfNoExists && !exists) {
                yield fs.promises.mkdir(pvsFolderPath);
            }
            else if (!exists) {
                return "";
            }
            return pvsFolderPath;
        });
    }
    static getSuppressionLogFilePath() {
        return __awaiter(this, void 0, void 0, function* () {
            const pvsFolderPath = yield this.getPvsFolderPath();
            return path.join(pvsFolderPath, this.suppressionLogFileName);
        });
    }
    static getSuppressionFilePaths() {
        return __awaiter(this, void 0, void 0, function* () {
            const pvsFolderPath = yield this.getPvsFolderPath();
            const pattern = new vscode.RelativePattern(pvsFolderPath, "*.suppress.json");
            const suppressFilePaths = (yield vscode.workspace.findFiles(pattern)).map(u => u.fsPath);
            let pathsOfCurrectFiles = [];
            let pathsOfIncorrectFiles = [];
            for (var p of suppressFilePaths) {
                try {
                    const suppressContent = fs.readFileSync(p, { encoding: "utf8" });
                    JSON.parse(suppressContent);
                    pathsOfCurrectFiles.push(p);
                }
                catch (_a) {
                    pathsOfIncorrectFiles.push(p);
                }
            }
            if (pathsOfIncorrectFiles.length > 0) {
                let message = 'Some suppression files have an incorrect format:';
                for (var p of pathsOfIncorrectFiles) {
                    message += "\n" + p;
                }
                MessageSender_1.MessageSender.send(message, "warning");
            }
            return pathsOfCurrectFiles;
        });
    }
    static getTargetSuppressionFilePath(baseSuppressionFileName, allSupressionFilePaths = undefined) {
        return __awaiter(this, void 0, void 0, function* () {
            allSupressionFilePaths !== null && allSupressionFilePaths !== void 0 ? allSupressionFilePaths : (allSupressionFilePaths = yield this.getSuppressionFilePaths());
            if (allSupressionFilePaths.length === 1) {
                return allSupressionFilePaths[0];
            }
            else if (allSupressionFilePaths.length > 1) {
                let selectFilePath = allSupressionFilePaths.find(p => path.basename(p) === `${baseSuppressionFileName}`);
                if (!selectFilePath) {
                    selectFilePath = allSupressionFilePaths[0];
                }
                return selectFilePath;
            }
            const pvsFolderPath = yield this.getPvsFolderPath();
            const selectFilePath = path.join(pvsFolderPath, `${baseSuppressionFileName}`);
            return selectFilePath;
        });
    }
    static isPathsEquals(first, second) {
        if (os.platform() === "win32") {
            return first.toLowerCase().replaceAll('\\', '/') === second.toLowerCase().replaceAll('\\', '/');
        }
        else {
            return first.replaceAll('\\', '/') === second.replaceAll('\\', '/');
        }
    }
}
exports.AnalyzerUtils = AnalyzerUtils;
AnalyzerUtils.analysisTimeoutValues = [
    { id: "After_10_minutes", value: 600, name: "After 10 minutes" },
    { id: "After_30_minutes", value: 1800, name: "After 30 minutes" },
    { id: "After_60_minutes", value: 3600, name: "After 60 minutes" },
    { id: "No_timeout", value: 999999, name: "No timeout" }
];
AnalyzerUtils.suppressionLogFileName = "Suppression.log";
AnalyzerUtils.warningsToSuppressionFileName = "WarningsToSuppression.json";
AnalyzerUtils.sriMappingUri = vscode.Uri.joinPath(vscode.extensions.getExtension("EvgeniyRyzhkov.pvs-studio-vscode").extensionUri, "webview-settings", "build", "data", "srimap.json");
AnalyzerUtils.alwaysEnabledDiagnostics = ["V009", "V061", "V062", "V063"];
//# sourceMappingURL=AnalyzerUtils.js.map