"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CMake = void 0;
const vscode = require("vscode");
const openLinkInBrowser_1 = require("../../utilities/openLinkInBrowser");
const CompileCommands_1 = require("./CompileCommands");
const fs = require("fs");
const path = require("path");
const AnalyzerUtils_1 = require("../AnalyzerUtils");
const saveFile_1 = require("../../utilities/saveFile");
const MessageSender_1 = require("../../utilities/MessageSender");
/** Класс для работы с API CMake плагина */
class CMake {
    constructor() {
        /** Идентификатор CMake Tools в vscode маркете */
        this._vscodeCmakeToolsId = "ms-vscode.cmake-tools";
        this._acceptableCMakeGenerators = [
            "Ninja",
            "Unix Makefiles"
        ];
    }
    getBuildDirectory() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isCmakeToolsInstalled()) {
                try {
                    const buildDirrectory = (yield vscode.commands.executeCommand("cmake.buildDirectory"));
                    vscode.commands.executeCommand("PVSPanel.focus");
                    if (buildDirrectory) {
                        return buildDirrectory;
                    }
                }
                catch (_a) { }
                MessageSender_1.MessageSender.send("There is no build directory. Please, build your project before analysis.", "error");
            }
        });
    }
    getCompileCommandsJsonPath(filteredPaths = []) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isCmakeToolsInstalled()) {
                return;
            }
            const buildDir = yield this.getBuildDirectory();
            if (!buildDir) {
                return;
            }
            this._compileCommands = new CompileCommands_1.CompileCommands(buildDir);
            if (!this._compileCommands.path) {
                yield this.generateCompileCommands().catch((error) => {
                    throw error;
                });
            }
            return filteredPaths.length > 0 ? yield this.getPathOfFilteredCompileCommands(this._compileCommands.path, filteredPaths)
                : this._compileCommands.path;
        });
    }
    getPathOfFilteredCompileCommands(compileCommandsPath, filteredPaths) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!compileCommandsPath) {
                return;
            }
            try {
                let compileCommands = JSON.parse((yield fs.promises.readFile(compileCommandsPath)).toString());
                compileCommands = compileCommands.filter((data) => filteredPaths.some(p => AnalyzerUtils_1.AnalyzerUtils.isPathsEquals(p, data.file)));
                const tempFolderPath = yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath();
                const filteredCompileCommandsPath = path.join(tempFolderPath, "filtered_compile_commands.json");
                const response = yield (0, saveFile_1.saveFile)(compileCommands, filteredCompileCommandsPath);
                if (response) {
                    MessageSender_1.MessageSender.send(response.message, "error");
                    return;
                }
                return filteredCompileCommandsPath;
            }
            catch (e) {
                MessageSender_1.MessageSender.send(e.message, "error");
            }
        });
    }
    isCmakeToolsInstalled() {
        const cmakeTools = vscode.extensions.getExtension(this._vscodeCmakeToolsId);
        if (!cmakeTools) {
            this.showErrorMessageIfNoCmake();
        }
        return cmakeTools ? true : false;
    }
    showErrorMessageIfNoCmake() {
        return __awaiter(this, void 0, void 0, function* () {
            const installCmakeAction = "Install";
            const answer = yield MessageSender_1.MessageSender.send("The \"CMake Tools\" extension must be installed to start the analysis.", "error", false, installCmakeAction);
            if (answer === installCmakeAction) {
                const url = `https://marketplace.visualstudio.com/items?itemName=ms-vscode.cmake-tools`;
                (0, openLinkInBrowser_1.default)(url);
            }
        });
    }
    generateCompileCommands() {
        return __awaiter(this, void 0, void 0, function* () {
            const cmakeSettings = vscode.workspace.getConfiguration("cmake");
            const userSettings = {
                exportCompileCommandsFile: cmakeSettings.exportCompileCommandsFile
            };
            this._compileCommands.updatePath();
            if (!this._compileCommands.path) {
                yield this.isAcceptableGenerator(cmakeSettings).catch(error => {
                    throw error;
                });
            }
            // Включаем генерацию .json
            yield cmakeSettings.update("exportCompileCommandsFile", true, vscode.ConfigurationTarget.Workspace);
            const code = yield vscode.commands.executeCommand("cmake.configure");
            vscode.commands.executeCommand("PVSPanel.focus");
            // Возвращаем настройки юзера
            yield cmakeSettings.update("exportCompileCommandsFile", userSettings.exportCompileCommandsFile, vscode.ConfigurationTarget.Workspace);
            this._compileCommands.updatePath();
            if (!this._compileCommands.path || code !== 0) {
                throw new Error("Failed to get compilation database from build system.");
            }
        });
    }
    isAcceptableGenerator(cmakeSettings) {
        return __awaiter(this, void 0, void 0, function* () {
            let generator = cmakeSettings.get("generator", "");
            const isAcceptableGenerator = this._acceptableCMakeGenerators.includes(generator);
            if (!isAcceptableGenerator) {
                let message = "The CMake generator is not assigned or supported by the analyzer." +
                    " To run the analysis, set the \"Generator\" parameter of the CMake Tools extension to one of the following values:";
                for (var i = 0; i < this._acceptableCMakeGenerators.length; i++) {
                    const gen = this._acceptableCMakeGenerators[i];
                    message += `\n- ${gen}`;
                    message += i === this._acceptableCMakeGenerators.length - 1 ? "." : ";";
                }
                const response = yield MessageSender_1.MessageSender.send(message, "error", true, "Edit setting");
                if (response === "Edit setting") {
                    vscode.commands.executeCommand("workbench.action.openSettings", "cmake.generator", 2);
                }
                throw new Error();
            }
        });
    }
}
exports.CMake = CMake;
//# sourceMappingURL=CMake.js.map