"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CMakeAnalyzer = void 0;
const Analyzer_1 = require("../Analyzer");
const vscode = require("vscode");
const CMake_1 = require("./CMake");
const Config_1 = require("./Config");
const fs = require("fs");
const CMakeAnalyzerOutput_1 = require("./CMakeAnalyzerOutput");
const Regedit_1 = require("../Regedit");
const path = require("path");
const child_process_1 = require("child_process");
const saveFile_1 = require("../../utilities/saveFile");
const AnalyzerUtils_1 = require("../AnalyzerUtils");
const PluginSettingsManager_1 = require("../../PluginSettingsManager");
const os = require("os");
const MessageSender_1 = require("../../utilities/MessageSender");
class CMakeAnalyzer extends Analyzer_1.AnalyzerWithLicenseActions {
    constructor() {
        super();
        this.description = {
            analyzerName: "CMake projects analyzer",
            analyzedProjectsType: "CMake (C, C++)",
            conditionForAnalysisPossibility: "There is a CmakeLists.txt file in the open directory",
            extensionsOfAnalyzedFiles: [".cpp", ".cc", ".cxx"],
            needToParseDirectories: true
        };
        this._analyzerIsNotSupportedMessage = "Current version of the PVS-Studio Analyzer is not supported. Please update the analyzer.";
        this.endAnalysisKey = "b0467edca75f";
        this.cmake = new CMake_1.CMake();
    }
    activateLicense(userName, licenseKey) {
        return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
            let analyzerOutput = [];
            const analyzerPath = yield this.getAnalyzerPath();
            const tempFolderPath = yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath();
            const licenseProcess = (0, child_process_1.spawn)(analyzerPath.path, ["credentials", userName, licenseKey], tempFolderPath ? {
                cwd: yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath()
            } : undefined);
            licenseProcess.stdout.on("data", (data) => {
                let dataStr = data.toString();
                if (dataStr.startsWith("Error:")) {
                    if (dataStr.includes("Non-zero exit code from C++ core.")) {
                        dataStr = "Incorrect registration info.";
                    }
                    MessageSender_1.MessageSender.send(dataStr, "error");
                }
                else {
                    MessageSender_1.MessageSender.send(dataStr, "info");
                }
                analyzerOutput.push(...dataStr.split(/\r\n?|\n\r?/gm));
            });
            licenseProcess.stderr.on("error", (error) => {
                MessageSender_1.MessageSender.send(error.message, "error");
            });
            licenseProcess.on("close", (code) => {
                // @ts-ignore
                resolve({ code, output: analyzerOutput });
            });
        }));
    }
    getLicenseInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                const analyzerPath = yield this.getAnalyzerPath();
                const tempFolderPath = yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath();
                const licenseProcess = (0, child_process_1.spawn)(analyzerPath.path, ["lic-info"], tempFolderPath ? {
                    cwd: yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath()
                } : undefined);
                const licenseInfo = [];
                let licenseFilePath = "";
                licenseProcess.stdout.on("data", (data) => {
                    const dataLines = data.toString().split(/\r\n?|\n\r?/gm);
                    let nextLineIsPath = false;
                    licenseInfo.push(...dataLines.map((line) => {
                        if (nextLineIsPath) {
                            licenseFilePath = line;
                            nextLineIsPath = false;
                        }
                        nextLineIsPath = line.startsWith("Path to file with license");
                        return line.trim();
                    }));
                });
                licenseProcess.on("close", _ => {
                    resolve({ licenseInfo, licenseFilePath });
                });
            }));
        });
    }
    getAnalyzerPath() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const platform = os.platform();
            const pathFromConfig = (_a = vscode.workspace.getConfiguration("pvsStudio").cmakeProjectsAnalyzer.path) === null || _a === void 0 ? void 0 : _a.trim();
            if (pathFromConfig && (!/^CompilerCommandsAnalyzer.exe$/.test(path.basename(pathFromConfig)) || !fs.existsSync(pathFromConfig))) {
                MessageSender_1.MessageSender.send(`The specified path to the ${this.description.analyzerName.toLowerCase()} is incorrect or points to a non-existent file.`, "error");
                return { path: pathFromConfig, isSupported: false };
            }
            else if (pathFromConfig) {
                return { path: pathFromConfig, isSupported: true };
            }
            if (platform === "win32") {
                const installDir = yield (new Regedit_1.Regedit).getInstallDir();
                if (installDir) {
                    if (!(yield this.checkAnalyzerVersionWithExecution(path.join(installDir, "x64", 'PVS-Studio.exe'), true, true))) {
                        MessageSender_1.MessageSender.send(this._analyzerIsNotSupportedMessage, "error");
                        return { path: path.join(installDir, "CompilerCommandsAnalyzer.exe"), isSupported: false };
                    }
                    return { path: path.join(installDir, "CompilerCommandsAnalyzer.exe"), isSupported: true };
                }
            }
            else {
                const command = "pvs-studio-analyzer";
                const checkCommandProcess = (0, child_process_1.spawn)("which", [command], {
                    cwd: yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath()
                });
                let path = "";
                yield new Promise((resolve) => {
                    checkCommandProcess.stdout.on("data", (stdout) => __awaiter(this, void 0, void 0, function* () {
                        path = stdout.toString().trim();
                        resolve();
                    }));
                    checkCommandProcess.on("close", (code, signal) => {
                        resolve();
                    });
                });
                if (path === "") {
                }
                if (!(yield this.checkAnalyzerVersionWithExecution('pvs-studio', false, true))) {
                    MessageSender_1.MessageSender.send(this._analyzerIsNotSupportedMessage, "error");
                    return { path: path, isSupported: false };
                }
                return { path: path, isSupported: true };
            }
            return { path: "", isSupported: true };
        });
    }
    initializeAnalysisProcess(reportPanel, onAnalysisStarted, isAnalysisEnded, analyzedFiles) {
        return __awaiter(this, void 0, void 0, function* () {
            let isIntermodular = yield this.selectIntermodularMode();
            if (isIntermodular === undefined) {
                return;
            }
            onAnalysisStarted();
            const requirementsForRun = yield this.getRequirementsForRun(analyzedFiles);
            if (!requirementsForRun || isAnalysisEnded()) {
                // FIXME: Что-то получше надо чем просто выйти
                return;
            }
            const { configPath, compileCommandsJsonPath, suppressionPaths } = requirementsForRun;
            const targetSuppressionFilePath = yield AnalyzerUtils_1.AnalyzerUtils.getTargetSuppressionFilePath("suppress_file.suppress.json", suppressionPaths);
            const pluginSettings = PluginSettingsManager_1.PluginSettingsManager.settings;
            const threadCount = pluginSettings.analysis.threadCount;
            const isDumpFiles = !pluginSettings.analysis.removeIntermediateFiles;
            const sourceTreeRoot = pluginSettings.sourceTreeRoot;
            let analyzerArgs = [
                "analyze",
                "--cfg",
                configPath,
                "--file",
                compileCommandsJsonPath,
                "--print-stdout",
                "--threads",
                `${threadCount}`
            ];
            if (fs.existsSync(targetSuppressionFilePath)) {
                analyzerArgs.push("-s", targetSuppressionFilePath);
            }
            if (sourceTreeRoot) {
                analyzerArgs.push("-r", sourceTreeRoot);
            }
            if (isDumpFiles) {
                analyzerArgs = [...analyzerArgs, "--dump-files"];
            }
            if (isIntermodular) {
                analyzerArgs.push("--intermodular");
            }
            const output = new CMakeAnalyzerOutput_1.CMakeAnalyzerOutput();
            if (suppressionPaths.length > 1) {
                const warningSeveralSuppresionFiles = AnalyzerUtils_1.AnalyzerUtils.toGridMessage('Several suppression files were found in the .PVS-Studio folder. Only one suppression file is supported at the moment. The file ${targetSuppressionFilePath} is used, the others are ignored.');
                output.setAnalysisOutput(JSON.stringify(warningSeveralSuppresionFiles));
            }
            const analyzerPath = (yield this.getAnalyzerPath());
            if (isAnalysisEnded()) {
                return;
            }
            let lastProgressValue = 0;
            let isAnalysisStage = false;
            const process = yield this.processAnalysis(analyzerPath.path, analyzerArgs, output, reportPanel.view, {
                cwd: requirementsForRun.buildDir
            }, (progress) => {
                if (isIntermodular) {
                    if (progress < lastProgressValue) {
                        isAnalysisStage = true;
                    }
                    lastProgressValue = progress;
                    return Math.ceil(isAnalysisStage ? 50 + progress / 2 : progress / 2);
                }
                else {
                    return progress;
                }
            });
            return process;
        });
    }
    selectIntermodularMode() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.intermodularAnalysisIsEnabled === undefined) {
                return new Promise(resolve => {
                    const quickPick = vscode.window.createQuickPick();
                    const intermodularItem = { label: "Intermodular analysis" };
                    const regularItem = { label: "Regular analysis" };
                    quickPick.title = "Select analysis mode";
                    quickPick.placeholder = "Intermodular analysis can take a longer time, but will give you a better results.";
                    quickPick.selectedItems = [intermodularItem];
                    quickPick.items = [intermodularItem, regularItem];
                    quickPick.onDidChangeSelection(e => {
                        if (e.length === 0) {
                            return;
                        }
                        if (e[0].label === intermodularItem.label) {
                            resolve(true);
                        }
                        else {
                            resolve(false);
                        }
                        quickPick.dispose();
                    });
                    quickPick.onDidHide(_ => {
                        quickPick.dispose();
                        resolve(undefined);
                    });
                    quickPick.show();
                    MessageSender_1.MessageSender.send("Please specify the analysis mode in the command palette.", "info");
                });
            }
            else {
                return this.intermodularAnalysisIsEnabled;
            }
        });
    }
    getRequirementsForRun(analyzedFiles) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const compileCommandsJsonPath = yield this.cmake.getCompileCommandsJsonPath(analyzedFiles);
                const cmakeBuildDir = yield this.cmake.getBuildDirectory();
                if (!cmakeBuildDir || !compileCommandsJsonPath) {
                    return;
                }
                this._config = new Config_1.Config(cmakeBuildDir);
                const configPath = yield this._config.getConfigPath();
                let suppressionPaths = yield AnalyzerUtils_1.AnalyzerUtils.getSuppressionFilePaths();
                const legacySuppressFile = suppressionPaths.find(p => path.basename(p, ".suppress.json") === "suppress_base");
                if (legacySuppressFile) {
                    const newSuppressFile = path.join(path.dirname(legacySuppressFile), "suppress_file.suppress.json");
                    suppressionPaths = suppressionPaths.filter(p => p !== legacySuppressFile);
                    suppressionPaths.push(newSuppressFile);
                    fs.promises.rename(legacySuppressFile, newSuppressFile);
                }
                return {
                    // @ts-ignore
                    compileCommandsJsonPath,
                    configPath,
                    suppressionPaths,
                    buildDir: cmakeBuildDir
                };
            }
            catch (error) {
                MessageSender_1.MessageSender.send(error.message, "error");
            }
        });
    }
    initializeSuppressionProcess(warnings, reportPanel) {
        return __awaiter(this, void 0, void 0, function* () {
            const buildDir = yield this.cmake.getBuildDirectory();
            if (!buildDir) {
                return;
            }
            const toSuppressionFilePath = yield AnalyzerUtils_1.AnalyzerUtils.getWarningsToSuppressionFilePath(); //path.join((await AnalyzerUtils.getPvsFolderPath()), "suppress_test.json");
            const reportForSuppression = { version: 2, warnings: warnings };
            const response = yield (0, saveFile_1.saveFile)(reportForSuppression, toSuppressionFilePath);
            if (response) {
                MessageSender_1.MessageSender.send(response.message, "error");
                return;
            }
            const targetSuppressionFilePath = yield AnalyzerUtils_1.AnalyzerUtils.getTargetSuppressionFilePath("suppress_file.suppress.json");
            let args = ["suppress", "-a", "all", "-o", targetSuppressionFilePath, toSuppressionFilePath];
            const errorLogFile = (yield AnalyzerUtils_1.AnalyzerUtils.getSuppressionLogFilePath());
            const output = new CMakeAnalyzerOutput_1.CMakeAnalyzerOutput();
            const analyzerPath = yield this.getAnalyzerPath();
            const process = yield this.processSuppression(analyzerPath.path, args, reportPanel.view, warnings, targetSuppressionFilePath, errorLogFile, { cwd: buildDir });
            process.stdout.on("data", data => {
                output.setSuppressionOutput(data);
            });
            process.on("close", _ => {
                if (output.suppressedCount !== warnings.length) {
                    MessageSender_1.MessageSender.send(`Not all of the selected warnings were suppressed. At the next analyzer run it is possible that already suppressed warnings will appear again.`, "warning");
                }
            });
            return process;
        });
    }
    isAnalysisPossible() {
        return __awaiter(this, void 0, void 0, function* () {
            const rootFolder = AnalyzerUtils_1.AnalyzerUtils.getRootFolder();
            if (!rootFolder) {
                return false;
            }
            return fs.existsSync(vscode.Uri.joinPath(rootFolder.uri, "CMakeLists.txt").fsPath);
        });
    }
    getUpdateLicenseArgs(userName, licenseKey) {
        return ["credentials", userName, licenseKey];
    }
}
exports.CMakeAnalyzer = CMakeAnalyzer;
//# sourceMappingURL=CMakeAnalyzer.js.map