"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JavaAnalyzerConfigManager = void 0;
const vscode = require("vscode");
const JavaAnalyzerConfig_1 = require("./JavaAnalyzerConfig");
const fs = require("fs");
const AnalyzerUtils_1 = require("../../AnalyzerUtils");
const stripJsonComments = require("strip-json-comments");
const path = require("path");
const MessageSender_1 = require("../../../utilities/MessageSender");
class JavaAnalyzerConfigManager {
    static tryGetAnalyzerConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            const configPath = yield this.getAnalyserRunConfigPath();
            let config;
            if (!fs.existsSync(configPath)) {
                const response = yield MessageSender_1.MessageSender.send(this._createAnalyserConfigFileMessage, "info", true, "Edit", "Continue");
                if (response !== undefined) {
                    config = new JavaAnalyzerConfig_1.JavaAnalyzerConfig();
                    yield this.createConfigFile(configPath, config, true);
                }
                else {
                    return;
                }
                if (response === "Edit") {
                    const runConfigPath = yield JavaAnalyzerConfigManager.getAnalyserRunConfigPath();
                    vscode.window.showTextDocument(vscode.Uri.file(runConfigPath));
                    return;
                }
            }
            else {
                const configText = yield fs.promises.readFile(configPath);
                try {
                    config = JSON.parse(stripJsonComments(configText.toString()));
                }
                catch (_a) {
                    MessageSender_1.MessageSender.send(this._invalidArgsFileFormatMessage.replace(`{runConfigPath}`, configPath), "error");
                    return;
                }
            }
            if (config["suppress-base"] && !(yield this.checkPathParam("suppress-base", config["suppress-base"], { fileType: "file" }))) {
                return;
            }
            const loggingPossibleValues = ["ALL", "TRACE", "DEBUG", "INFO", "WARN", "ERROR", "OFF"];
            if (config["logging"] && !loggingPossibleValues.includes(config["logging"])) {
                let message = "An incorrect value of the logging parameter is specified. Acceptable values:";
                for (var i = 0; i < loggingPossibleValues.length; i++) {
                    message += ` ${loggingPossibleValues[i]}`;
                    message += i === loggingPossibleValues.length - 1 ? "." : ",";
                }
                MessageSender_1.MessageSender.send(message, "error");
                return;
            }
            return config;
        });
    }
    static checkPathParam(paramName, pathValue, options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (options) {
                const fileName = path.basename(pathValue);
                if ((options === null || options === void 0 ? void 0 : options.checkExists) && !fs.existsSync(pathValue)) {
                    MessageSender_1.MessageSender.send(`The path passed to the ${paramName} parameter is incorrect or points to a non-existent target.`, "error");
                    return false;
                }
                if (options.fileNamePattern && !options.fileNamePattern.test(fileName)) {
                    MessageSender_1.MessageSender.send(`The "${pathValue}" path passed to "${paramName}" parameter does not match the expected one. Please read the description of the parameter.`, "error");
                    return false;
                }
                if (options.fileType) {
                    const type = yield fs.promises.stat(pathValue);
                    if (options.fileType === "directory" && !type.isDirectory()) {
                        MessageSender_1.MessageSender.send(`The "${pathValue}" path specified in the "${paramName}" parameter should point to the directory.`, "error");
                        return false;
                    }
                    else if (options.fileType === "file" && !type.isFile()) {
                        MessageSender_1.MessageSender.send(`The "${pathValue}" path specified in the "${paramName}" parameter should point to the file.`, "error");
                        return false;
                    }
                }
            }
            return true;
        });
    }
    static getAnalyserRunConfigPath() {
        return __awaiter(this, void 0, void 0, function* () {
            return path.join((yield AnalyzerUtils_1.AnalyzerUtils.getPvsFolderPath()), this._analyserRunConfigName);
        });
    }
    static createConfigFile(runConfigPath, config, withDiscription = false) {
        return __awaiter(this, void 0, void 0, function* () {
            let configJson = JSON.stringify(config, undefined, 2);
            if (withDiscription) {
                configJson += `\n\n\n${(0, JavaAnalyzerConfig_1.getParamDescriptionComment)()}`;
            }
            if (fs.existsSync(runConfigPath)) {
                yield fs.promises.truncate(runConfigPath, 0);
            }
            yield fs.promises.writeFile(runConfigPath, configJson);
        });
    }
}
exports.JavaAnalyzerConfigManager = JavaAnalyzerConfigManager;
JavaAnalyzerConfigManager._analyserRunConfigName = "JavaAnalyzerConfig.jsonc";
JavaAnalyzerConfigManager._invalidArgsFileFormatMessage = `The file with analysis parameters is incorrect. Check: {runConfigPath}.`;
JavaAnalyzerConfigManager._createAnalyserConfigFileMessage = "A new file with analysis parameters was created.\n\nDo you want to edit it (after running the analysis again) or continue with the default parameters?";
//# sourceMappingURL=JavaAnalyzerConfigManager.js.map