"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JavaAnalyzer = void 0;
const Analyzer_1 = require("../Analyzer");
const JavaAnalyzerConfigManager_1 = require("./AnalyzerConfig/JavaAnalyzerConfigManager");
const JavaAnalyzerOutput_1 = require("./JavaAnalyzerOutput");
const AnalyzerUtils_1 = require("../AnalyzerUtils");
const path = require("path");
const fs = require("fs");
const vscode = require("vscode");
const PluginSettingsManager_1 = require("../../PluginSettingsManager");
const saveFile_1 = require("../../utilities/saveFile");
const JavaProjectManager_1 = require("./JavaProjectManager");
const os = require("os");
const child_process_1 = require("child_process");
const MessageSender_1 = require("../../utilities/MessageSender");
/* TODO:
  В rawjson Java ядра есть дополнительные поля, которые не сохраняются после обработки в плагине. Соответственно в сохраняемом отчете их уже не будет.
 */
class JavaAnalyzer extends Analyzer_1.AnalyzerWithLicenseActions {
    constructor() {
        super(...arguments);
        this.description = {
            analyzerName: "Java projects analyzer",
            analyzedProjectsType: "Java",
            conditionForAnalysisPossibility: "The folder is a root folder of a Java project (the project must be loaded with the \"Project manager for Java\" extension)",
            extensionsOfAnalyzedFiles: [".java"],
            needToParseDirectories: false
        };
        this.endAnalysisKey = "63a517f9";
        this._minAnalyzerVersion = "7.28.77080";
        this._supressionFileName = "suppress_base.json";
        this._tempReportForSupressionName = "ReportForSupression.json";
        this._tempExtFileName = "ExternalsForJavaAnalysis.txt";
        this._tempAnalyzerConfigFileName = "ArgsForJavaAnalysis.json";
        //readonly _createAnalyserConfigFileMessage = "A new file with analysis parameters was created.\n\nDo you want to edit it (after running the analysis again) or continue with the default parameters?";
        this._analyzerIsNotSupportedMessage = "Current version of the PVS-Studio Java Analyzer is not supported. Please update the analyzer.";
        this._specifiedPathToAnalyzerIsIncorrectMessage = "The specified path to PVS-Studio Java Analyzer is incorrect or points to a non-existent file.";
        this._jdkNotBeFoundMessage = "The JDK could not be found. Make sure it is installed. In this case, you can explicitly specify the path to it in the settings.";
        this._analyzerDirPattern = /^\d{1}\.\d{1,2}\.\d{1,5}$/;
        this._newLinePattern = /\r\n?|\n\r?/gm;
    }
    isAnalysisPossible() {
        return __awaiter(this, void 0, void 0, function* () {
            const rootFolder = AnalyzerUtils_1.AnalyzerUtils.getRootFolder();
            if (!rootFolder) {
                return false;
            }
            let count = 3;
            let callback = () => __awaiter(this, void 0, void 0, function* () {
                const result = yield JavaProjectManager_1.JavaProjectManager.loadJavaProjectsFromWorkspace();
                if (result === undefined && count > 0) {
                    count = count - 1;
                    yield new Promise(resolve => setTimeout(resolve, 1000));
                    return yield callback();
                }
                else {
                    return result !== null && result !== void 0 ? result : false;
                }
            });
            return callback();
            //return await JavaProjectManager.loadJavaProjectsFromWorkspace(outputEnable);
        });
    }
    onRunningExtension() {
        return __awaiter(this, void 0, void 0, function* () {
            JavaProjectManager_1.JavaProjectManager.createSourceWatcher(this.description.extensionsOfAnalyzedFiles);
            JavaProjectManager_1.JavaProjectManager.createOrUpdateProjectFile(this.description.extensionsOfAnalyzedFiles, false, () => false);
        });
    }
    activateLicense(userName, licenseKey) {
        return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
            let analyzerOutput = new Set();
            const analyzerPath = yield this.getAnalyzerPath();
            let jdkPath = yield this.tryGetJDKPath();
            if (!jdkPath) {
                return;
            }
            const licenseProcess = (0, child_process_1.spawn)(jdkPath, ["-jar", analyzerPath.path, "--activate-license", "--user-name", userName, "--license-key", licenseKey], {
                cwd: yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath()
            });
            licenseProcess.stdout.on("data", (data) => {
                const dataStr = data.toString();
                for (var line of dataStr.split(/\r\n?|\n\r?/gm).filter(line => line.trim() !== "")) {
                    analyzerOutput.add(line);
                }
            });
            licenseProcess.stderr.on("error", (error) => {
                for (var line of error.message.split(/\r\n?|\n\r?/gm).filter(line => line.trim() !== "")) {
                    analyzerOutput.add(line);
                }
            });
            licenseProcess.on("close", (code) => {
                let outputArr = Array.from(analyzerOutput);
                outputArr = outputArr.filter(line => {
                    if (line.startsWith("Analysis finished")) {
                        return false;
                    }
                    return true;
                });
                // @ts-ignore
                resolve({ code, output: outputArr });
            });
        }));
    }
    getLicenseInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                const analyzerPath = yield this.getAnalyzerPath();
                let jdkPath = yield this.tryGetJDKPath();
                if (!jdkPath) {
                    return;
                }
                const tempFolderPath = yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath();
                const licenseProcess = (0, child_process_1.spawn)(jdkPath, ["-jar", analyzerPath.path, "--check-license"], tempFolderPath ? {
                    cwd: yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath()
                } : undefined);
                const licenseInfo = [];
                let output = "";
                licenseProcess.stdout.on("data", (data) => {
                    output += data.toString();
                });
                licenseProcess.on("close", _ => {
                    let nextIsPath = false;
                    let licenseFilePath = "";
                    for (var line of output.split(this._newLinePattern)) {
                        line = line.trim();
                        if (line === "") {
                            continue;
                        }
                        try {
                            const jsonObj = JSON.parse(line);
                            const daysCount = Math.floor((jsonObj.timeout - Date.now() / 1000) / (3600 * 24));
                            licenseInfo.push(`License type: ${jsonObj.type ? jsonObj.type : "Invalid license"}.`, `The license will expire in ${daysCount.toString()} days.`);
                        }
                        catch (_a) {
                            if (line.toLowerCase() === "path to file with license information:") {
                                licenseInfo.push(line);
                                nextIsPath = true;
                            }
                            else if (nextIsPath) {
                                licenseFilePath = line;
                                licenseInfo.push(line);
                                nextIsPath = false;
                            }
                        }
                    }
                    resolve({ licenseInfo, licenseFilePath });
                });
            }));
        });
    }
    getAnalyzerPath() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const platform = os.platform();
            let analyzerPath = (_a = vscode.workspace.getConfiguration("pvsStudio").javaProjectsAnalyzer.path) === null || _a === void 0 ? void 0 : _a.trim();
            try {
                const java = yield this.tryGetJDKPath();
                if (!java) {
                    return { path: "", isSupported: false };
                }
                if (!(yield this.checkJdkVersion(java))) {
                    return { path: "", isSupported: false };
                    ;
                }
                else if (analyzerPath) {
                    if ((!/pvs-studio(.jar)?$/.test(path.basename(analyzerPath)) || !fs.existsSync(analyzerPath))) {
                        MessageSender_1.MessageSender.send(this._specifiedPathToAnalyzerIsIncorrectMessage, "error");
                        return { path: "", isSupported: false };
                    }
                    const analyzerVersion = yield this.getAnalyzerVersion(analyzerPath, java);
                    if (!analyzerVersion || !(yield this.checkAnalyzerVersion(analyzerVersion))) {
                        MessageSender_1.MessageSender.send(this._analyzerIsNotSupportedMessage, "error");
                        return { path: "", isSupported: false };
                    }
                }
                else {
                    let defaultPath = platform === "win32" ? path.join(process.env.APPDATA, "PVS-Studio-Java") : path.join(process.env.HOME, ".config", "PVS-Studio-Java");
                    if (fs.existsSync(defaultPath)) {
                        let directories = (yield fs.promises.readdir(defaultPath)).map(n => path.join(defaultPath, n)).filter(p => fs.lstatSync(p).isDirectory());
                        let selectAnalyzerPath = "";
                        let selectAnalyzerVersionNum = -1;
                        let isInstalled = false;
                        for (var dir of directories) {
                            const tempAnalyzerPath = path.join(dir, "pvs-studio.jar");
                            if (!fs.existsSync(tempAnalyzerPath)) {
                                continue;
                            }
                            isInstalled = true;
                            const analyzerVersion = yield this.getAnalyzerVersion(tempAnalyzerPath, java);
                            if (!analyzerVersion) {
                                continue;
                            }
                            const { isSupported, isDebug } = yield this.checkAnalyzerVersion(analyzerVersion);
                            const tempAnalyzerVersionNum = Number(analyzerVersion.join(""));
                            if (isDebug || isSupported && (!selectAnalyzerPath || tempAnalyzerVersionNum > selectAnalyzerVersionNum)) {
                                selectAnalyzerPath = tempAnalyzerPath;
                                selectAnalyzerVersionNum = tempAnalyzerVersionNum;
                            }
                            if (isDebug) {
                                break;
                            }
                        }
                        if (isInstalled && !selectAnalyzerPath) {
                            MessageSender_1.MessageSender.send(this._analyzerIsNotSupportedMessage, "error");
                            return { path: "", isSupported: false };
                        }
                        analyzerPath = selectAnalyzerPath;
                    }
                }
            }
            catch (TypeError) {
                return { path: "", isSupported: false };
            }
            return { path: analyzerPath, isSupported: true };
        });
    }
    getAnalyzerVersion(analyzerPath, javaPath) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                const tempFolderPath = yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath();
                const versionProcess = (0, child_process_1.spawn)(javaPath, ["-jar", analyzerPath, "--version"], tempFolderPath ? {
                    cwd: yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath()
                } : undefined);
                versionProcess.stdout.on("data", data => {
                    var _a, _b;
                    const dataStr = data.toString().replaceAll(/\r\n?|\n\r?/gm, "");
                    // @ts-ignore
                    const versionStr = (_b = (_a = /(?<version>\d+\.\d+\.\d+)/.exec(dataStr)) === null || _a === void 0 ? void 0 : _a.groups) === null || _b === void 0 ? void 0 : _b.version;
                    if (!versionStr) {
                        return;
                    }
                    const versionChunks = versionStr.split(".").map(chunk => Number(chunk));
                    resolve(versionChunks);
                });
                versionProcess.on("close", _ => {
                    resolve(undefined);
                });
            }));
        });
    }
    initializeAnalysisProcess(reportPanel, onAnalysisStarted, isAnalysisEnded, analyzedFiles) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const analyzerPath = yield this.getAnalyzerPath();
            const rootFolderPath = AnalyzerUtils_1.AnalyzerUtils.getRootFolder().uri.fsPath;
            const analyzerConfig = yield this.tryGetAnalyzerConfig(analyzedFiles);
            onAnalysisStarted();
            if (!analyzerConfig) {
                return;
            }
            const tempFolder = yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath();
            const tempConfigPath = path.join(tempFolder, this._tempAnalyzerConfigFileName);
            const error = yield (0, saveFile_1.saveFile)(analyzerConfig, tempConfigPath);
            if (error) {
                return;
            }
            const projectFileIsCreated = yield JavaProjectManager_1.JavaProjectManager.createOrUpdateProjectFile(this.description.extensionsOfAnalyzedFiles, true, isAnalysisEnded);
            if (!projectFileIsCreated) {
                return;
            }
            const suppressFilePath = yield AnalyzerUtils_1.AnalyzerUtils.getTargetSuppressionFilePath(this._supressionFileName);
            let jvmArguments = vscode.workspace.getConfiguration("pvsStudio").javaProjectsAnalyzer.jvmArguments;
            const args = [...jvmArguments, "-jar", analyzerPath.path, "-p", rootFolderPath, "-t", "json", "--cfg", tempConfigPath, "--suppress-base", suppressFilePath, "--writeToStdout"];
            const output = new JavaAnalyzerOutput_1.JavaAnalyzerOutput();
            if (isAnalysisEnded()) {
                return;
            }
            const process = yield this.processAnalysis((_a = analyzerConfig["java-path"]) !== null && _a !== void 0 ? _a : "java", args, output, reportPanel.view);
            process.stdout.on("data", chunk => {
                if (chunk.includes("Unknown option: '--writeToStdout'")) {
                    MessageSender_1.MessageSender.send(this._analyzerIsNotSupportedMessage, "error");
                }
            });
            process.on("close", (_) => __awaiter(this, void 0, void 0, function* () {
                if (PluginSettingsManager_1.PluginSettingsManager.settings.analysis.removeIntermediateFiles) {
                    const pvsFolderPath = yield AnalyzerUtils_1.AnalyzerUtils.getPvsFolderPath();
                    const additionalTempFiles = (yield fs.promises.readdir(pvsFolderPath)).filter(name => name === "temp.json" ||
                        name === "temp_cmd_args.json").map(name => path.join(pvsFolderPath, name));
                    for (var p of additionalTempFiles) {
                        yield fs.promises.unlink(p);
                    }
                }
            }));
            return process;
        });
    }
    checkJdkVersion(jdkPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const javaVersion = yield this.getJdkVersion(jdkPath);
            if (javaVersion) {
                if (javaVersion > 19) {
                    MessageSender_1.MessageSender.send("PVS-Studio does not fully support the analysis of projects with the JDK version greater than 19. Analysis results may be incorrect.", "warning");
                    return true;
                }
                if (javaVersion < 11) {
                    MessageSender_1.MessageSender.send("PVS-Studio does not support the analysis of projects with the JDK version lower than 11.", "error");
                    return false;
                }
                return true;
            }
            return false;
        });
    }
    tryGetAnalyzerConfig(filesForAnalysis) {
        return __awaiter(this, void 0, void 0, function* () {
            const config = yield JavaAnalyzerConfigManager_1.JavaAnalyzerConfigManager.tryGetAnalyzerConfig();
            if (!config) {
                return;
            }
            const pluginSettings = PluginSettingsManager_1.PluginSettingsManager.settings;
            config["timeout"] = pluginSettings.analysis.timeout.value / 60;
            config["threads"] = pluginSettings.analysis.threadCount;
            config["verbose"] = !pluginSettings.analysis.removeIntermediateFiles;
            if (pluginSettings.sourceTreeRoot) {
                config["sourcetree-root"] = pluginSettings.sourceTreeRoot;
            }
            config["disabled-warnings"] = pluginSettings.disabledDiagnostics.map(d => d.code);
            if (!config["suppress-base"] || config["suppress-base"] === "") {
                const suppressFilePath = yield AnalyzerUtils_1.AnalyzerUtils.getTargetSuppressionFilePath(this._supressionFileName);
                if (fs.existsSync(suppressFilePath)) {
                    config["suppress-base"] = suppressFilePath;
                }
            }
            let javaPath = yield this.tryGetJDKPath();
            if (javaPath !== undefined) {
                config["java-path"] = javaPath;
            }
            if (filesForAnalysis && filesForAnalysis.length > 0) {
                config["analyze-only"] = filesForAnalysis;
            }
            config["launch-mode"] = "VSCODE";
            return config;
        });
    }
    tryGetJDKPath() {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            let javaPath = (_b = (_a = vscode.workspace.getConfiguration("pvsStudio")) === null || _a === void 0 ? void 0 : _a.javaProjectsAnalyzer.javaPath) === null || _b === void 0 ? void 0 : _b.trim();
            if (javaPath) {
                if (!fs.existsSync(javaPath) || !/java(.exe)?/.test(path.basename(javaPath))) {
                    MessageSender_1.MessageSender.send("The path passed to the \"java Project Analyzer.javaPath\" setting is incorrect or points to a non-existent target.", "error");
                    return;
                }
                return javaPath;
            }
            else {
                const javaHome = (_d = (_c = vscode.workspace.getConfiguration("java")) === null || _c === void 0 ? void 0 : _c.jdt.ls.java.home) === null || _d === void 0 ? void 0 : _d.trim();
                if (javaHome) {
                    return path.join(javaHome, "bin", "java");
                }
                const hasJavaInSystemPaths = (yield this.getJdkVersion("java")) !== undefined;
                if (!hasJavaInSystemPaths) {
                    new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                        const response = yield MessageSender_1.MessageSender.send(this._jdkNotBeFoundMessage, "error", false, "Go to settings");
                        if (response === "Go to settings") {
                            vscode.commands.executeCommand("workbench.action.openSettings", "pvsStudio.javaProjectsAnalyzer.javaPath");
                        }
                        resolve();
                    }));
                    return;
                }
                else {
                    return "java";
                }
            }
        });
    }
    initializeSuppressionProcess(warnings, reportPanel) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const reportForSuppression = { version: 2, warnings: warnings };
            const warningsToSupressionFilePath = yield AnalyzerUtils_1.AnalyzerUtils.getWarningsToSuppressionFilePath();
            yield (0, saveFile_1.saveFile)(reportForSuppression, warningsToSupressionFilePath);
            const analyzerPath = yield this.getAnalyzerPath();
            const analyzerConfig = yield this.tryGetAnalyzerConfig();
            if (!analyzerConfig) {
                return;
            }
            const suppressionFilePath = (_a = analyzerConfig["suppress-base"]) !== null && _a !== void 0 ? _a : yield AnalyzerUtils_1.AnalyzerUtils.getTargetSuppressionFilePath(this._supressionFileName);
            let jvmArguments = vscode.workspace.getConfiguration("pvsStudio").javaProjectsAnalyzer.jvmArguments;
            const args = [...jvmArguments,
                "-jar", analyzerPath.path,
                "--convert", "toSuppress",
                "--src-convert", warningsToSupressionFilePath,
                "--dst-convert", suppressionFilePath];
            const sourceTreeRoot = PluginSettingsManager_1.PluginSettingsManager.settings.sourceTreeRoot;
            if (sourceTreeRoot) {
                args.push("--sourcetree-root", sourceTreeRoot);
            }
            const logFilePath = yield AnalyzerUtils_1.AnalyzerUtils.getSuppressionLogFilePath();
            const analyzerJavaPath = yield this.tryGetJDKPath();
            if (!analyzerJavaPath) {
                return;
            }
            const process = yield this.processSuppression(!analyzerJavaPath ? "java" : analyzerJavaPath, args, reportPanel.view, warnings, suppressionFilePath, logFilePath);
            return process;
        });
    }
    getJdkVersion(jdkPath) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                const checkProcess = (0, child_process_1.spawn)(jdkPath, ["-version"], {
                    cwd: yield AnalyzerUtils_1.AnalyzerUtils.getTempFolderPath()
                });
                checkProcess.on("error", (_) => {
                    resolve(undefined);
                });
                let versionNumber = -1;
                const callback = (line) => {
                    var _a, _b;
                    if (versionNumber >= 0) {
                        return;
                    }
                    // @ts-ignore
                    const version = (_b = (_a = /version "(?<version>\d+\.\d+)/.exec(line)) === null || _a === void 0 ? void 0 : _a.groups) === null || _b === void 0 ? void 0 : _b.version;
                    if (version) {
                        const versionChunks = version.split(".").map(str => Number(str));
                        versionNumber = versionChunks[0] === 1 ? versionChunks[1] : versionChunks[0];
                    }
                };
                if (!checkProcess) {
                    resolve(undefined);
                    return;
                }
                checkProcess.stdout.on("data", chunk => {
                    for (var line of chunk.toString().split(this._newLinePattern).filter((str) => str.length > 0)) {
                        callback(line);
                    }
                });
                checkProcess.stderr.on("data", chunk => {
                    for (var line of chunk.toString().split(this._newLinePattern).filter((str) => str.length > 0)) {
                        callback(line);
                    }
                });
                checkProcess.on("close", _ => {
                    if (versionNumber < 0) {
                        resolve(undefined);
                    }
                    else {
                        resolve(versionNumber);
                    }
                });
            }));
        });
    }
}
exports.JavaAnalyzer = JavaAnalyzer;
//# sourceMappingURL=JavaAnalyzer.js.map