"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JavaProjectManager = exports.NodeKind = void 0;
const vscode = require("vscode");
const path = require("path");
const AnalyzerUtils_1 = require("../AnalyzerUtils");
const fs = require("fs");
const saveFile_1 = require("../../utilities/saveFile");
const MessageSender_1 = require("../../utilities/MessageSender");
var NodeKind;
(function (NodeKind) {
    NodeKind[NodeKind["workspace"] = 1] = "workspace";
    NodeKind[NodeKind["project"] = 2] = "project";
    NodeKind[NodeKind["packageRoot"] = 3] = "packageRoot";
    NodeKind[NodeKind["package"] = 4] = "package";
    NodeKind[NodeKind["primaryType"] = 5] = "primaryType";
    NodeKind[NodeKind["compilationUnit"] = 6] = "compilationUnit";
    NodeKind[NodeKind["classFile"] = 7] = "classFile";
    NodeKind[NodeKind["container"] = 8] = "container";
    NodeKind[NodeKind["folder"] = 9] = "folder";
    NodeKind[NodeKind["file"] = 10] = "file";
})(NodeKind = exports.NodeKind || (exports.NodeKind = {}));
class ExecuteOptions {
    constructor() {
        this.isWorkspaceCommand = true;
    }
}
class JavaProjectManager {
    static createOrUpdateProjectFile(validSourceExtensions, outputEnable, isAnalysisEnded) {
        return __awaiter(this, void 0, void 0, function* () {
            const needToReloadExternals = !this._projectFileIsBeingCreated;
            while (this._projectFileIsBeingCreated) {
                if (isAnalysisEnded()) {
                    return false;
                }
                yield new Promise(resolve => setTimeout(resolve, 1000));
            }
            this._projectFileIsBeingCreated = true;
            const pvsFolderPath = yield AnalyzerUtils_1.AnalyzerUtils.getPvsFolderPath();
            const projectFilePath = path.join(pvsFolderPath, this._projectFileName);
            let projectFileObj = undefined;
            if (fs.existsSync(projectFilePath)) {
                try {
                    const projectFileJSON = (yield fs.promises.readFile(projectFilePath)).toString();
                    projectFileObj = JSON.parse(projectFileJSON);
                }
                catch (_a) { }
            }
            const nonAnalyzedProjectPaths = [];
            const modules = [];
            if (isAnalysisEnded()) {
                this._projectFileIsBeingCreated = false;
                return false;
            }
            try {
                for (var projectUri of this._allProjectsUriList) {
                    const { module, isCanceled } = yield this.createOrUpdateModule(projectUri, projectFileObj, validSourceExtensions, needToReloadExternals, isAnalysisEnded);
                    if (isCanceled) {
                        this._projectFileIsBeingCreated = false;
                        return false;
                    }
                    if (!module) {
                        nonAnalyzedProjectPaths.push(vscode.Uri.parse(projectUri).fsPath);
                        continue;
                    }
                    modules.push(module);
                }
            }
            catch (_b) {
                this._projectFileIsBeingCreated = false;
                return false;
            }
            if (modules.length === 0) {
                if (outputEnable) {
                    MessageSender_1.MessageSender.send("Failed to evaluate projects.", "error");
                }
                this._projectFileIsBeingCreated = false;
                return false;
            }
            const nonAnalyzedProjectsLogPath = path.join(pvsFolderPath, this._nonAnalyzedProjectsLog);
            let error = yield (0, saveFile_1.saveFile)(nonAnalyzedProjectPaths, nonAnalyzedProjectsLogPath);
            if (nonAnalyzedProjectPaths.length > 0 && !error && outputEnable) {
                new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                    const message = `PVS-Studio is unable to get dependency or source information for some projects. These projects will not be analyzed.`;
                    const response = yield MessageSender_1.MessageSender.send(message, "warning", false, "Go to the list");
                    if (response === "Go to the list") {
                        vscode.window.showTextDocument(vscode.Uri.file(nonAnalyzedProjectsLogPath));
                    }
                    resolve();
                }));
            }
            error = yield (0, saveFile_1.saveFile)({ modules: modules }, projectFilePath);
            if (error) {
                this._projectFileIsBeingCreated = false;
                return false;
            }
            if (!this._projectFileHasBeenRecreated) {
                this._projectFileHasBeenRecreated = true;
            }
            else {
                this._changedSourcesOrDirectories.clear();
            }
            this._projectFileIsBeingCreated = false;
            return true;
        });
    }
    static loadJavaProjectsFromWorkspace() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isTargetExtensionInstalledAndActived()) {
                return false;
            }
            const projectsUri = yield JavaProjectManager.getProjectsUriFromWorkspace(true);
            if (!projectsUri) {
                return;
            }
            this._allProjectsUriList = projectsUri;
            return this._allProjectsUriList.length > 0;
        });
    }
    static createSourceWatcher(validSourceExtensions) {
        let sourcePattern = "{";
        for (var i = 0; i < validSourceExtensions.length; i++) {
            sourcePattern += validSourceExtensions[i];
            sourcePattern += i === validSourceExtensions.length - 1 ? '}' : ",";
        }
        const callback = (p) => {
            const pExt = path.extname(p);
            if (validSourceExtensions.some(ext => pExt === ext) || pExt === "" && !p.split(path.sep).some(chunk => chunk.startsWith("."))) {
                this._changedSourcesOrDirectories.add(p);
            }
        };
        const relativePattern = new vscode.RelativePattern(AnalyzerUtils_1.AnalyzerUtils.getRootFolder().uri.fsPath, `**/*`);
        this._watcher = vscode.workspace.createFileSystemWatcher(relativePattern);
        this._watcher.onDidChange(e => callback(e.fsPath));
        this._watcher.onDidCreate(e => callback(e.fsPath));
        this._watcher.onDidDelete(e => callback(e.fsPath));
    }
    static createOrUpdateModule(projectUri, oldProjectFileObj, validSourceExtensions, needToReloadExternals, isAnalysisEnded) {
        return __awaiter(this, void 0, void 0, function* () {
            const rootFolderPath = AnalyzerUtils_1.AnalyzerUtils.getRootFolder().uri.fsPath;
            const projectPath = vscode.Uri.parse(projectUri).fsPath;
            const oldModule = oldProjectFileObj === null || oldProjectFileObj === void 0 ? void 0 : oldProjectFileObj.modules.find(m => m.path === projectPath);
            if (!this._projectFileHasBeenRecreated || !oldModule || !projectPath.startsWith(rootFolderPath)) {
                return this.createModule(projectUri, validSourceExtensions, oldModule, isAnalysisEnded);
            }
            else {
                return this.updateModule(projectUri, validSourceExtensions, oldModule, needToReloadExternals, rootFolderPath, isAnalysisEnded);
            }
        });
    }
    static updateModule(projectUri, validSourceExtensions, oldModule, needToReloadExternals, rootFolderPath, isAnalysisEnded) {
        return __awaiter(this, void 0, void 0, function* () {
            const newModule = oldModule;
            const response = yield this.executeCommand(this._getPackageDataCommand, undefined, { kind: NodeKind.project, projectUri: projectUri });
            if (isAnalysisEnded()) {
                return { module: undefined, isCanceled: true };
            }
            if (response.isError) {
                return { module: undefined, isCanceled: false };
            }
            if (needToReloadExternals) {
                const externals = yield this.getExternals(projectUri);
                if (isAnalysisEnded()) {
                    return { module: undefined, isCanceled: true };
                }
                if (!externals) {
                    return { module: undefined, isCanceled: false };
                }
                newModule.classpath = externals;
            }
            else {
                newModule.classpath = oldModule.classpath;
            }
            const childNodes = response.result;
            newModule.sources = newModule.sources.filter(s => s.startsWith(rootFolderPath));
            newModule.testSources = newModule.testSources.filter(s => s.startsWith(rootFolderPath));
            newModule.generatedSources = newModule.generatedSources.filter(s => s.startsWith(rootFolderPath));
            for (var childNode of childNodes) {
                if (!childNode.uri) {
                    continue;
                }
                let nodePath = vscode.Uri.parse(childNode.uri).fsPath;
                if (nodePath.endsWith(path.sep)) {
                    nodePath = nodePath.substring(0, nodePath.length - 1);
                }
                // if (path.basename(nodePath).startsWith("spring-websocket")) {
                //   console.log("ok");
                // }
                if (!nodePath.startsWith(rootFolderPath)) {
                    const result = yield this.getAllSourcesFromNode(childNode, validSourceExtensions, isAnalysisEnded);
                    if (result.isCanceled) {
                        return { module: undefined, isCanceled: true };
                    }
                    newModule.sources.push(...result.regularSources);
                    newModule.testSources.push(...result.testSources);
                    newModule.generatedSources.push(...result.generatedSources);
                }
                else {
                    for (var changedItem of this._changedSourcesOrDirectories) {
                        if (isAnalysisEnded()) {
                            return { module: undefined, isCanceled: true };
                        }
                        if (changedItem.endsWith(path.sep)) {
                            changedItem = changedItem.substring(0, changedItem.length - 1);
                        }
                        const changeItemUri = vscode.Uri.file(changedItem).toString();
                        if (!changedItem.startsWith(nodePath) && !this._allProjectsUriList.some(uri => uri.startsWith(changeItemUri))) {
                            continue;
                        }
                        const existsItem = fs.existsSync(changedItem);
                        if (!existsItem) {
                            newModule.sources = newModule.sources.filter(s => !s.startsWith(changedItem));
                            newModule.testSources = newModule.testSources.filter(s => !s.startsWith(changedItem));
                            newModule.generatedSources = newModule.generatedSources.filter(s => !s.startsWith(changedItem));
                        }
                        else {
                            if (childNode.kind === NodeKind.folder &&
                                changedItem === nodePath &&
                                this._generatedFoldersName.some(n => n === path.basename(changedItem)) &&
                                !newModule.generatedSources.includes(changedItem)) {
                                newModule.generatedSources.push(changedItem);
                                if (!newModule.sources.includes(changedItem)) {
                                    newModule.sources.push(changedItem);
                                }
                            }
                            else {
                                const stat = yield fs.promises.stat(changedItem);
                                const sources = stat.isDirectory() ? yield this.getSourcePathsFromDirectoryRecursive(changedItem, validSourceExtensions, isAnalysisEnded)
                                    : [changedItem];
                                if (sources === undefined) {
                                    return { module: undefined, isCanceled: true };
                                }
                                for (var source of sources) {
                                    if (isAnalysisEnded()) {
                                        return { module: undefined, isCanceled: true };
                                    }
                                    if (!newModule.sources.includes(source)) {
                                        newModule.sources.push(source);
                                    }
                                    const response = yield this.executeCommand(this._isTestFileCommand, undefined, vscode.Uri.file(source).toString());
                                    if (response.isError) {
                                        continue;
                                    }
                                    const testSourceIndex = newModule.testSources.indexOf(source);
                                    if (response.result && testSourceIndex < 0) {
                                        newModule.testSources.push(source);
                                    }
                                    else if (!response.result && testSourceIndex >= 0) {
                                        newModule.testSources.splice(testSourceIndex, 1);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            newModule.sources = Array.from(new Set(newModule.sources));
            newModule.testSources = Array.from(new Set(newModule.testSources));
            newModule.generatedSources = Array.from(new Set(newModule.generatedSources));
            return { module: newModule, isCanceled: false };
        });
    }
    static createModule(projectUri, validSourceExtensions, oldModule, isAnalysisEnded) {
        return __awaiter(this, void 0, void 0, function* () {
            const regularSourcePaths = [];
            const testSourcePaths = [];
            const generatedSourcePaths = [];
            let externalsPaths = [];
            if (isAnalysisEnded()) {
                return { module: undefined, isCanceled: true };
            }
            const response = yield this.executeCommand(this._getPackageDataCommand, undefined, { kind: NodeKind.project, projectUri: projectUri });
            if (response.isError) {
                return { module: undefined, isCanceled: false };
            }
            const childNodes = response.result;
            if (isAnalysisEnded()) {
                return { module: undefined, isCanceled: true };
            }
            const externals = yield this.getExternals(projectUri);
            if (!externals) {
                return { module: undefined, isCanceled: false };
            }
            externalsPaths = externals;
            for (var childNode of childNodes) {
                if (isAnalysisEnded()) {
                    return { module: undefined, isCanceled: true };
                }
                const result = yield this.getAllSourcesFromNode(childNode, validSourceExtensions, isAnalysisEnded);
                if (result.isCanceled) {
                    return { module: undefined, isCanceled: true };
                }
                regularSourcePaths.push(...result.regularSources);
                testSourcePaths.push(...result.testSources);
                generatedSourcePaths.push(...result.generatedSources);
            }
            const module = {
                name: path.basename(vscode.Uri.parse(projectUri).fsPath),
                path: vscode.Uri.parse(projectUri).fsPath,
                sources: regularSourcePaths,
                testSources: testSourcePaths,
                generatedSources: generatedSourcePaths,
                classpath: externalsPaths
            };
            return { module, isCanceled: false };
        });
    }
    static getAllSourcesFromNode(node, validSourceExtensions, isAnalysisEnded) {
        return __awaiter(this, void 0, void 0, function* () {
            const regularSources = [];
            const testSources = [];
            const generatedSources = [];
            const nodeUri = this.getUriFromNode(node);
            if (!nodeUri) {
                return { isCanceled: false, regularSources, testSources, generatedSources };
            }
            const nodePath = vscode.Uri.parse(nodeUri).fsPath;
            if (node.kind === NodeKind.folder && ["build", "bin", "target"].includes(path.basename(nodePath))) {
                regularSources.push(nodePath);
                generatedSources.push(nodePath);
            }
            else {
                const sourceFiles = [];
                const nodeExt = path.extname(nodePath);
                if ((node.kind === NodeKind.primaryType || node.kind === NodeKind.file) && validSourceExtensions.some(ext => nodeExt === ext)) {
                    sourceFiles.push(nodePath);
                }
                else if (node.kind === NodeKind.packageRoot || node.kind === NodeKind.folder && !path.basename(nodePath).startsWith(".") && !this._allProjectsUriList.some(uri => uri.startsWith(nodeUri))) {
                    const sources = yield this.getSourcePathsFromDirectoryRecursive(nodePath, validSourceExtensions, isAnalysisEnded);
                    if (!sources) {
                        return { isCanceled: true, regularSources, testSources, generatedSources };
                    }
                    sourceFiles.push(...sources);
                }
                for (var p of sourceFiles) {
                    if (isAnalysisEnded()) {
                        return { isCanceled: true, regularSources, testSources, generatedSources };
                    }
                    const response = yield this.executeCommand(this._isTestFileCommand, undefined, vscode.Uri.file(p).toString());
                    if (!response.isError && response.result) {
                        testSources.push(p);
                    }
                    regularSources.push(p);
                }
            }
            return { isCanceled: false, regularSources, testSources, generatedSources };
        });
    }
    static getSourcePathsFromDirectoryRecursive(directoryPath, validSourceExtensions, isAnalysisEnded) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (isAnalysisEnded()) {
                return;
            }
            const sourcePaths = [];
            try {
                const folderItemPaths = (yield fs.promises.readdir(directoryPath)).map(name => path.join(directoryPath, name));
                const files = [];
                const directories = [];
                for (var p of folderItemPaths) {
                    const stat = yield fs.promises.stat(p);
                    if (stat.isDirectory()) {
                        directories.push(p);
                    }
                    else if (stat.isFile()) {
                        files.push(p);
                    }
                }
                sourcePaths.push(...files.filter(p => validSourceExtensions.includes(path.extname(p))));
                for (var d of directories) {
                    sourcePaths.push(...(_a = yield this.getSourcePathsFromDirectoryRecursive(d, validSourceExtensions, isAnalysisEnded)) !== null && _a !== void 0 ? _a : []);
                }
            }
            catch (_b) { }
            return sourcePaths;
        });
    }
    static getUriFromNode(node) {
        let uri;
        if (node.uri) {
            uri = vscode.Uri.parse(node.uri).toString();
        }
        else if (node.path) {
            if (path.isAbsolute(node.path)) {
                uri = vscode.Uri.file(node.path).toString();
            }
            else {
                uri = vscode.Uri.file(path.resolve(AnalyzerUtils_1.AnalyzerUtils.getRootFolder().uri.fsPath, node.path)).toString();
            }
        }
        return uri;
    }
    static getProjectsUriFromWorkspace(excludeDefaultProject = false) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            let response = yield this.executeCommand(this._projectsUriFindCommand);
            if (response.isError) {
                return undefined;
            }
            let projects_1 = (_b = (_a = response.result) === null || _a === void 0 ? void 0 : _a.map((uri) => vscode.Uri.parse(uri).toString())) !== null && _b !== void 0 ? _b : [];
            if (projects_1 && excludeDefaultProject) {
                projects_1 = projects_1.filter(uri => {
                    const p = vscode.Uri.parse(uri).fsPath;
                    return path.basename(p) !== "jdt.ls-java-project";
                });
            }
            const rootFolderUri = AnalyzerUtils_1.AnalyzerUtils.getRootFolder().uri.toString();
            response = yield this.executeCommand(this._getProjectsListCommand, undefined, rootFolderUri);
            if (response.isError) {
                return undefined;
            }
            const projects_2 = (_d = (_c = response.result) === null || _c === void 0 ? void 0 : _c.map((projectNode) => vscode.Uri.parse(projectNode.uri).toString())) !== null && _d !== void 0 ? _d : [];
            return [...projects_1, ...projects_2.filter((uri_2) => !projects_1.some((uri_1) => uri_2.startsWith(uri_1)))];
        });
    }
    static getExternals(projectUri) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield this.executeCommand(this._getClasspathsCommand, undefined, projectUri, JSON.stringify({ scope: "runtime" }));
            if (response.isError) {
                return;
            }
            const externals = [...response.result.classpaths, ...response.result.modulepaths];
            return externals;
        });
    }
    static executeCommand(command, options, ...args) {
        return __awaiter(this, void 0, void 0, function* () {
            options !== null && options !== void 0 ? options : (options = new ExecuteOptions());
            try {
                if (options.isWorkspaceCommand === true) {
                    const response = yield vscode.commands.executeCommand(this._executeWorkspaceCommand, command, ...args);
                    return { isError: false, result: response };
                }
                else {
                    const response = yield vscode.commands.executeCommand(command, ...args);
                    return { isError: false, result: response };
                }
            }
            catch (e) {
                return { isError: true, result: undefined };
            }
        });
    }
    static isTargetExtensionInstalledAndActived() {
        const extension = vscode.extensions.getExtension(this._tagetJavaExtensionId);
        if (!extension) {
            return { isInstalled: false, isActived: false };
        }
        else if (!extension.isActive) {
            return { isInstalled: true, isActived: false };
        }
        return { isInstalled: true, isActived: true };
    }
}
exports.JavaProjectManager = JavaProjectManager;
JavaProjectManager._projectFileName = "project.json";
JavaProjectManager._nonAnalyzedProjectsLog = "nonAnalyzedProjects.log";
JavaProjectManager._targetJavaExtensionName = "Project Manager for Java";
JavaProjectManager._tagetJavaExtensionId = "vscjava.vscode-java-dependency";
// private static _targetJavaExtensionNotActivedMessage = `The \"${this._targetJavaExtensionName}\" extension must be actived.`;
// private static _targetJavaExtensionNotInstalledMessage = `The \"${this._targetJavaExtensionName}\" extension must be installed.`;
// private static _javaExtensionsIsLoadMessage = `"The "${this._targetJavaExtensionName}" extension loads java projects in the workspace. Please run the analysis at the end of this process.`;
JavaProjectManager._generatedFoldersName = ["build", "bin", "target"];
JavaProjectManager._executeWorkspaceCommand = "java.execute.workspaceCommand";
//private static _updateConfigurationCommand = "java.projectConfiguration.update";
// private static _updateMavenConfigurationCommand = "maven.java.projectConfiguration.update";
// private static _updateGradleConfigurationCommand = "gradle.java.projectConfiguration.update";
JavaProjectManager._projectsUriFindCommand = "java.project.getAll";
JavaProjectManager._getProjectsListCommand = "java.project.list";
JavaProjectManager._getClasspathsCommand = "java.project.getClasspaths";
JavaProjectManager._getPackageDataCommand = "java.getPackageData";
JavaProjectManager._isTestFileCommand = "java.project.isTestFile";
JavaProjectManager._changedSourcesOrDirectories = new Set();
JavaProjectManager._projectFileHasBeenRecreated = false;
JavaProjectManager._projectFileIsBeingCreated = false;
//# sourceMappingURL=JavaProjectManager.js.map