"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SettingsPanel = void 0;
const vscode = require("vscode");
const changeObjectProp_1 = require("../utilities/changeObjectProp");
const getJsonFromFile_1 = require("../utilities/getJsonFromFile");
const getUri_1 = require("../utilities/getUri");
const pathNode = require("path");
const AnalyzerManager_1 = require("../analyzer/AnalyzerManager");
const PluginSettingsManager_1 = require("../PluginSettingsManager");
const AnalyzerUtils_1 = require("../analyzer/AnalyzerUtils");
function getWebviewOptions(extensionUri) {
    return {
        // Enable javascript in the webview
        enableScripts: true,
        // @ts-ignore
        retainContextWhenHidden: true,
    };
}
class SettingsPanel {
    static createOrShow(extensionUri, params) {
        const column = vscode.window.activeTextEditor
            ? vscode.window.activeTextEditor.viewColumn
            : undefined;
        // If we already have a panel, show it.
        if (SettingsPanel.currentPanel) {
            SettingsPanel.currentPanel._panel.reveal(column);
            if (params && params.elemToShow) {
                SettingsPanel.currentPanel._panel.webview.postMessage({
                    type: "showElem",
                    elemToShow: params.elemToShow,
                });
            }
            return;
        }
        // Otherwise, create a new panel.
        const panel = vscode.window.createWebviewPanel(SettingsPanel.viewType, "PVS-Studio Settings", vscode.ViewColumn.Active, getWebviewOptions(extensionUri));
        SettingsPanel.currentPanel = new SettingsPanel(panel, extensionUri, params);
    }
    static revive(panel, extensionUri) {
        SettingsPanel.currentPanel = new SettingsPanel(panel, extensionUri);
    }
    constructor(panel, extensionUri, params) {
        this._disposables = [];
        this._panel = panel;
        this._extensionUri = extensionUri;
        // Set the webview's initial html content
        this._update();
        // Listen for when the panel is disposed
        // This happens when the user closes the panel or when the panel is closed programmatically
        this._panel.onDidDispose(() => this.dispose(), null, this._disposables);
        // Update the content based on view changes
        this._panel.onDidChangeViewState((e) => {
            if (this._panel.visible) {
                this._update();
            }
        }, null, this._disposables);
        // Handle messages from the webview
        this._panel.webview.onDidReceiveMessage((message) => __awaiter(this, void 0, void 0, function* () {
            switch (message.command) {
                case "alert":
                    vscode.window.showErrorMessage(message.text);
                    return;
                case "getBuildUriAndSettings":
                    // The build folder from the React build output
                    const buildUri = (0, getUri_1.getUri)(this._panel.webview, extensionUri, [
                        "webview-settings",
                        "build",
                    ]);
                    this.buildUri = buildUri;
                    const settings = PluginSettingsManager_1.PluginSettingsManager.settings;
                    this._panel.webview.postMessage({
                        type: "buildUriAndSettings",
                        buildUri,
                        settings: settings,
                        params,
                        fsSeparator: pathNode.sep,
                        analysisTimeoutValues: AnalyzerUtils_1.AnalyzerUtils.analysisTimeoutValues
                    });
                    return;
                case "loadSettingsFile":
                    const loadedSettings = yield this.loadSettingsFile();
                    if (!loadedSettings) {
                        return;
                    }
                    PluginSettingsManager_1.PluginSettingsManager.saveSettings(loadedSettings, true);
                    return;
                case "changeSettings":
                    const { setting, value } = message;
                    this.changeSettings(setting, value);
                    return;
                case "saveSettings":
                    yield this.saveSettings();
                    return;
                case "removeExcludedPath": {
                    const settings = PluginSettingsManager_1.PluginSettingsManager.settings;
                    const { path } = message;
                    const newSettings = Object.assign(Object.assign({}, settings), { excludedPaths: settings.excludedPaths.filter((p) => !AnalyzerUtils_1.AnalyzerUtils.isPathsEquals(p, path)) });
                    PluginSettingsManager_1.PluginSettingsManager.saveSettings(newSettings);
                    return;
                }
                case "excludePath": {
                    this.excludePath(message.path);
                    return;
                }
                case "getSourceTreeRootViaExplorer": {
                    this.getSourceTreeRootViaExplorer();
                    return;
                }
                case "activateLicense": {
                    this.activateLicense(message.license);
                    return;
                }
                case "securityRelatedIssues": {
                    this.securityRelatedIssues(message);
                    return;
                }
            }
        }), null, this._disposables);
        PluginSettingsManager_1.PluginSettingsManager.addChangeEventListener(settings => {
            if (!this._panel) {
                return;
            }
            this._panel.webview.postMessage({
                type: "buildUriAndSettings",
                buildUri: this.buildUri,
                settings: settings,
                fsSeparator: pathNode.sep,
                analysisTimeoutValues: AnalyzerUtils_1.AnalyzerUtils.analysisTimeoutValues
            });
        });
    }
    dispose() {
        SettingsPanel.currentPanel = undefined;
        // Clean up our resources
        this._panel.dispose();
        while (this._disposables.length) {
            const x = this._disposables.pop();
            if (x) {
                x.dispose();
            }
        }
    }
    changeSettings(setting, value) {
        const settings = PluginSettingsManager_1.PluginSettingsManager.settings;
        const settingsChanged = (0, changeObjectProp_1.changeObjectProp)(settings, setting, value);
        PluginSettingsManager_1.PluginSettingsManager.saveSettings(settingsChanged);
    }
    activateLicense(license) {
        return __awaiter(this, void 0, void 0, function* () {
            const updateLicenseResult = yield AnalyzerManager_1.AnalyzerManager.updateLicense(license.name, license.key);
            if (!updateLicenseResult) {
                return;
            }
            this._panel.webview.postMessage({
                type: "licenseStatus",
                licenseStatusArr: updateLicenseResult.output
            });
        });
    }
    updateLicenseInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            const licenseInfo = yield AnalyzerManager_1.AnalyzerManager.getLicenseInfo();
            if (!licenseInfo) {
                return;
            }
            yield this._panel.webview.postMessage({
                type: "licenseInfoFromFile",
                licenseInfo: {
                    name: licenseInfo.userName,
                    key: licenseInfo.licenseKey,
                },
            });
            yield this._panel.webview.postMessage({
                type: "licenseStatus",
                licenseStatusArr: licenseInfo.data
            });
        });
    }
    updateLicenseStatus(license) {
        return __awaiter(this, void 0, void 0, function* () {
            const updateLicenseResult = yield AnalyzerManager_1.AnalyzerManager.updateLicense(license.name, license.key);
            if (!updateLicenseResult) {
                return;
            }
            this._panel.webview.postMessage({
                type: "licenseStatus",
                licenseStatusArr: updateLicenseResult.output
            });
        });
    }
    loadSettingsFile() {
        return __awaiter(this, void 0, void 0, function* () {
            // Выбираем файл через окно выбора файлов
            const fileObj = yield vscode.window.showOpenDialog();
            // Если файл не выбран
            if (!fileObj) {
                return;
            }
            // Отправляем сообщение "Загрузка началась"
            // if (this._view) {
            //   this._view.webview.postMessage({ type: "isLoading", isLoading: true });
            // }
            // Получаем путь до файла
            const path = fileObj[0].fsPath;
            // Проверяем, .json ли это
            if (!path.endsWith("json")) {
                vscode.window.showErrorMessage("The settings file should be in json format");
                return;
            }
            const settings = yield (0, getJsonFromFile_1.getJsonFromFile)(path);
            return settings;
        });
    }
    securityRelatedIssues(message) {
        return __awaiter(this, void 0, void 0, function* () {
            const settings = PluginSettingsManager_1.PluginSettingsManager.settings;
            const settingsChanged = (0, changeObjectProp_1.changeObjectProp)(settings, ['analysis', message.command], message.sriEnabled);
            PluginSettingsManager_1.PluginSettingsManager.saveSettings(settingsChanged);
            return settings.analysis.securityRelatedIssues;
        });
    }
    saveSettings() {
        return __awaiter(this, void 0, void 0, function* () {
            const saveFileInfo = yield vscode.window.showSaveDialog({
                filters: {
                    "JSON": ["json"]
                }
            });
            const settings = PluginSettingsManager_1.PluginSettingsManager.settings;
            let path = saveFileInfo === null || saveFileInfo === void 0 ? void 0 : saveFileInfo.fsPath;
            if (!path) {
                return;
            }
            if (!path.includes(".json")) {
                path += ".json";
            }
            yield PluginSettingsManager_1.PluginSettingsManager.saveSettings(settings, false, path);
        });
    }
    excludePath(path) {
        return __awaiter(this, void 0, void 0, function* () {
            const settings = PluginSettingsManager_1.PluginSettingsManager.settings;
            let excludedPaths = settings.excludedPaths || [];
            excludedPaths = [...new Set([...excludedPaths, path])];
            const newSettings = Object.assign(Object.assign({}, settings), { excludedPaths });
            PluginSettingsManager_1.PluginSettingsManager.saveSettings(newSettings);
        });
    }
    /** Получает Source Tree Root через окно выбора директории */
    getSourceTreeRootViaExplorer() {
        return __awaiter(this, void 0, void 0, function* () {
            // Выбираем файл через окно выбора файлов
            const strDirObj = yield vscode.window.showOpenDialog({
                canSelectFiles: false,
                canSelectFolders: true,
            });
            // Если файл не выбран
            if (!strDirObj) {
                return;
            }
            // Получаем путь до файла
            const path = strDirObj[0].fsPath;
            const settings = PluginSettingsManager_1.PluginSettingsManager.settings;
            const settingsChanged = (0, changeObjectProp_1.changeObjectProp)(settings, ["sourceTreeRoot"], path);
            PluginSettingsManager_1.PluginSettingsManager.saveSettings(settingsChanged);
        });
    }
    _update() {
        const webview = this._panel.webview;
        this._panel.webview.html = this._getHtmlForWebview(webview);
        this.updateLicenseInfo();
    }
    _getHtmlForWebview(webview) {
        // The CSS file from the React build output
        const stylesUri = (0, getUri_1.getUri)(webview, this._extensionUri, [
            "webview-settings",
            "build",
            "static",
            "css",
            "main.css",
        ]);
        // The JS file from the React build output
        const scriptUri = (0, getUri_1.getUri)(webview, this._extensionUri, [
            "webview-settings",
            "build",
            "static",
            "js",
            "main.js",
        ]);
        const codiconsUri = (0, getUri_1.getUri)(webview, this._extensionUri, [
            "webview-settings",
            "node_modules",
            "@vscode/codicons",
            "dist",
            "codicon.css",
        ]);
        // Use a nonce to only allow a specific script to be run.
        return /*html*/ `
      <!DOCTYPE html>
      <html lang="en">
        <head>
          <meta charset="utf-8">
          <meta name="viewport" content="width=device-width,initial-scale=1,shrink-to-fit=no">
          <meta name="theme-color" content="#000000">
          <link rel="stylesheet" type="text/css" href="${stylesUri}">
          <link rel="stylesheet" type="text/css" href="${codiconsUri}">
          <title>PVS-Studio Settings</title>
        </head>
        <body>
          <noscript>You need to enable JavaScript to run this app.</noscript>
          <div id="root"></div>
          <script src="${scriptUri}"></script>
        </body>
      </html>
    `;
    }
}
exports.SettingsPanel = SettingsPanel;
SettingsPanel.viewType = "PVS-Settings";
//# sourceMappingURL=SettingsPanel.js.map